/****************************************************************************
    qtrtti.h  11/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#ifndef _qtrtti_
#define _qtrtti_

class TypeInfo;
class OrdinalInfo;
class EnumInfo;
class ClassInfo;
class PropInfo;
class QObject;


#include <qstring.h>
#include <qstrlist.h>
#include <qevent.h>

#define QListT  QList

typedef enum {tkundef, tkord, tkstring, tkqstring, tkenum, tkset, tkclass, tksignal} TypeKind;

class TypeInfo {
public:
	const char	*name;
	TypeKind	kind;
	TypeInfo(const char *, TypeKind );
};

typedef QObject* (*CreateProc)(QObject*);
typedef void (*DefineProps)(ClassInfo *);

class ClassInfo: public TypeInfo {
public:
	ClassInfo	*next;
	PropInfo	*props;
	CreateProc	create;
	ClassInfo	*ancestor;
	ClassInfo(const char *nm, ClassInfo *bc, CreateProc c, DefineProps p);
static	ClassInfo* Find(const char*);
static	ClassInfo* Find(QObject*);
static  ClassInfo* Ancestor(const char*);
static  ClassInfo* Ancestor(QObject *);
	bool	inherits(const char*);
	PropInfo	*Prop(const char*);
};

class PropInfo {
public:
	PropInfo	*next;
	ClassInfo	*owner;
	char		*name;
	TypeInfo	*type;
	unsigned long 	getf, getd, setf, setd;
	PropInfo(ClassInfo *, TypeInfo *, char *, ...);
};

class OrdinalInfo : public TypeInfo {
public:
	int	size, min, max;
	OrdinalInfo(const char *nm, int n, int x, int s);
};

class EnumPair;

class EnumInfo : public OrdinalInfo {
public:
	EnumPair	*names;
	EnumInfo(const char *, int , ... );
};

class EnumPair {
public:
	EnumPair *next;
	char	 *name;
	int	 value;
	EnumPair(EnumInfo &info, char *nm, int v);
};

class SetInfo : public OrdinalInfo {
public:
	EnumInfo	*base;
	SetInfo(EnumInfo *, char *);
};


class SignalInfo: public TypeInfo {
public:
	SignalInfo(const char *);
};

class QSignalProp {
public:
	void	 *Receiver;
	QString	 Slot;	
	QSignalProp() {}
	QSignalProp(void *R, const char *S): Receiver(R), Slot(S) {} 
};

class QSignalList: public QListT<QSignalProp> {
public:
	QSignalList(bool autodelete) { setAutoDelete(autodelete); }
};

class QGetPosEvent: public QEvent {
public:
	QGetPosEvent(int t): QEvent((QEvent::Type) (QEvent::User + t)) {}
	int	X;
	int	Y;
};


extern "C" unsigned long getOrdProp(void*, PropInfo*);
extern "C" void setOrdProp(void*, PropInfo*, unsigned long value);

extern "C" QString getStrProp(void*, PropInfo*);
extern "C" void setStrProp(void*, PropInfo*, const char*);

extern "C" QSignalProp getSignalProp(void*, PropInfo*);
extern "C" void setSignalProp(void*, PropInfo*, QSignalProp );

extern "C" QObject *ReadFormTemplate(QString File, QObject *Obj);
extern "C" QObject *ReadFormResource(unsigned int *, QObject *);

extern "C" void WriteObjectTemplate(QString File, QObject *Obj); 

extern "C" QObject *FindChild(QObject *Top, const char *Name);

extern "C" bool getConnections(QObject *, QString, QObject *,
                                  QStrList, QSignalList &);

extern "C" bool getSlots(QObject *, QStrList &);

extern "C" bool getSignals(QObject *, QStrList &);





extern OrdinalInfo   	_char;
extern OrdinalInfo	_short;
extern OrdinalInfo 	_int;
extern EnumInfo		_bool;
extern TypeInfo		_string;
extern SignalInfo 	_signal;

extern ClassInfo _QObject;

extern ClassInfo _QFont;
extern ClassInfo _QColor;
extern ClassInfo _QStrList;
extern ClassInfo _QPixmap;
extern ClassInfo _QWidget;
extern ClassInfo _QButton;
extern ClassInfo _QDialog;
extern ClassInfo _QPushButton;
extern ClassInfo _QLineEdit;
extern ClassInfo _QLabel;
extern ClassInfo _QMultiLineEdit;
extern ClassInfo _QListBox;
extern ClassInfo _QComboBox;
extern ClassInfo _QFrame;
extern ClassInfo _QMenuBar;
extern ClassInfo _KTMainWindow;


#endif



