/****************************************************************************
    menueditor.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/
#define INCLUDE_MENUITEM_DEF   1

#include "menueditor.h"
#include <qpainter.h>
#include <qfontmetrics.h>
#include "../inspector/inspector.h"
#include "../formedit/designer.h"
#include <qtrtti.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <stdio.h>

int eindex = 0;

void Enter(const char *d)
{
   eindex ++;
   char tmp[256];
   memset(tmp, 32, 256);
   sprintf(tmp + eindex, "Enter %s\n", d); 
}

void Leave(const char *d)
{
   char tmp[256];
   memset(tmp, 32, 256);
   sprintf(tmp + eindex, "Leave %s\n", d); 
   eindex--;
}


class MenuDataHelper: public QMenuData 
{
public:
	QListT<QMenuItem>  *Items() { return (QListT<QMenuItem>  *) mitems; }

};

class   QMenuInfo;

class  QMenuLabel {
public:
	QRect     pos;
	QString	  text;
	QPixmap   pixmap;
	QMenuInfo *popup;
	QMenuInfo *parent;
	bool	  checked;
	bool	  enabled;
	QString   clicked;
	QMenuLabel(QMenuInfo*, QMenuItem *);
	QMenuLabel(QMenuInfo*);
	~QMenuLabel();
	QPixmap   &getPixmap() { return pixmap; }
	const     char *getText() { return text; }
	void	  setPixmap(QPixmap &);
	void      setText(const char *);
	void	  setClicked(QSignalProp);
	QSignalProp getClicked();
};

class	QMenuInfo {
public:
	bool		popup;
	int		pixmaped;
	QRect		pos;
	QMenuLabel	*parent;
	QListT<QMenuLabel>      labels;
	QMenuInfo(QMenuData *, bool Popup);
	QMenuInfo(QMenuLabel *);
	~QMenuInfo();
};

QListT<QMenuInfo>	popups;
QMenuInfo		*menuData = NULL;
QMenuLabel		*active = NULL;
MenuEditor		*editor;

QStrList		Slots;
QSignalList 		Connections(true);


QMenuLabel::QMenuLabel(QMenuInfo *P, QMenuItem *item)
: popup(0)
{
	parent = P;
	text = item->text();
	if (item->pixmap()) pixmap = *item->pixmap(); 
	checked = item->isChecked(); 		
	enabled = item->isEnabled(); 		
	if (item->isSeparator()) text = "-";

	if (getConnections((QObject*) item->signal(), "2x()", editor->designer, Slots, Connections))
		clicked = Connections.first()->Slot;

	if (item->popup()) popup = new QMenuInfo(item->popup(), true);
}

QMenuLabel::QMenuLabel(QMenuInfo *P)
: popup(0)
{
	parent = P;
	text = "";
	checked = false;
	enabled = false;
}

QMenuLabel::~QMenuLabel()
{ 
	if (active == this) 
	{
		InspectorDataList List;
		objectInspector->EditProperties(editor->designer, NULL, List);
		active = NULL;
	}	
	if (parent) 
	{
		parent->labels.take(parent->labels.find(this)); 
	}
	if (popup) delete popup;
} 


extern "C" void  ArrangeMenu(QPainter &p, QMenuInfo &menu, int x, int y);
extern "C" void  ReArrangeMenu(QMenuInfo &menu);


void QMenuLabel::setPixmap(QPixmap &Pixmap)
{
   pixmap = Pixmap;
   
   ReArrangeMenu(*parent);
   editor->repaint(parent->pos, true);   
   
   editor->checkLastItem(this);
} 

void QMenuLabel::setText(const char *Text)
{
   text = Text;

   ReArrangeMenu(*parent);
   editor->repaint(parent->pos, true);   

   editor->checkLastItem(this);
} 

void QMenuLabel::setClicked(QSignalProp  Value) 
{
	clicked = Value.Slot;
}

QSignalProp QMenuLabel::getClicked()
{
	return QSignalProp(NULL, clicked);
}


QMenuInfo::QMenuInfo(QMenuData *d, bool Popup)
{
	parent = NULL;
	popups.append(this);

	popup = Popup;
	
	QListT<QMenuItem>  	*items = ((MenuDataHelper*) d)->Items();

	if (items)
	for (items->first(); items->current(); items->next())
	{
		labels.append(new QMenuLabel(this, items->current()));
	}
// avoid empty menus 
	if (!labels.last()) 
	{
		labels.append(new QMenuLabel(this));
		return;
	}
	// allways add empty item at last
	labels.append(new QMenuLabel(this));
}

QMenuInfo::QMenuInfo(QMenuLabel *Parent)
{
	parent = Parent;
	popups.append(this);

	popup = true;
	
	labels.append(new QMenuLabel(this));
}

QMenuInfo::~QMenuInfo()
{ 
	if (menuData == this) menuData = NULL;
	if (parent) parent->popup = NULL;
	popups.take(popups.find(this)); 
	while (labels.first()) delete labels.first();
} 


void  ArrangeMenu(QPainter &p, QMenuInfo &menu, int x, int y)
{
	menu.pos = QRect(x, y, 0, 0);
	menu.pixmaped = 0;
	int  w = 0, h = 0;
	for (QMenuLabel *l = menu.labels.first(); l; l = menu.labels.next())
	{
		int lw = 0, lh = 0;
		
		if (!l->pixmap.isNull()) menu.pixmaped = QMAX(menu.pixmaped, l->pixmap.width());
		
		// es un popup o menubar
		if (menu.popup)
		{
			lw = p.fontMetrics().width(l->text) + 
			     (l->pixmap.isNull() ? 0 : l->pixmap.width());
    
                } else
                {
                	lw = l->pixmap.isNull() ? 
                	     p.fontMetrics().width(l->text) : 
                	     l->pixmap.width();
                
                }	
                if (l->text == "" && l->pixmap.isNull()) lw = QMAX(25, lw);
                lw += 8;
                lh = 8 + QMAX(p.fontMetrics().height(), l->pixmap.isNull() ? 0 : l->pixmap.height());
     
     		l->pos = QRect(x, y, lw, lh);
     		if (menu.popup) 
     		{
     			w = QMAX(w, lw);
	     		h += lh;
	     		y += lh;
	     	} else
	     	{
	     		w += lw;
	     		x += lw;
	     		h = QMAX(h, lh);
	     	}
	}
	x = menu.pos.x();
	y = menu.pos.y();
	menu.pos = QRect(x, y, w, h);
	
	for (QMenuLabel *l = menu.labels.first(); l; l = menu.labels.next())
	{
		// es un popup o menubar
		if (menu.popup)
		{
			l->pos.setWidth(menu.pos.width());
			if (l->popup) ArrangeMenu(p, *l->popup, l->pos.x()+l->pos.width(), l->pos.y());
                } else
                {
                	l->pos.setHeight(menu.pos.height());
			if (l->popup) ArrangeMenu(p, *l->popup, l->pos.x(), l->pos.y() + l->pos.height());
                }	
	}
}


void  ReArrangeMenu(QMenuInfo &menu)
{
	QPainter p(editor);
	ArrangeMenu(p, menu, menu.pos.x(), menu.pos.y());
}


void  fillMenuInfo(QMenuData *d, bool Popup)
{
	int x = 0, y = 0;
	if (menuData) delete menuData;
	popups.clear();
	menuData = new QMenuInfo(d, Popup);
	QPainter p(editor);
	ArrangeMenu(p, *menuData, x, y);
}

void Activate(QMenuInfo  *item)
{
	int	index = popups.find(item);
	popups.take(index);
	popups.append(item);
}

bool LabelAt(int x, int y, QMenuLabel **Item)
{
	for (QMenuInfo *popup = popups.last(); popup; popup = popups.prev())
	if (popup->pos.contains(QPoint(x, y), true))
	for (QMenuLabel *l=popup->labels.first(); l; l = popup->labels.next())
	if (l->pos.contains(QPoint(x, y), true))
	{
		if (Item) *Item = l;
		return true;
	}
	return false;
}

MenuEditor::MenuEditor(FormDesigner *P, QObject *o, QMenuData *Data, bool Popup)
: QMainWindow(NULL), designer(P), obj(o), menu(Data)
{
	initMetaObject();
	initRCData();

	editor = this;
	((MenuEditor*)designer->Widget())->setWFlags(WState_Disabled);
	active = NULL;
	getSlots(designer, Slots);
	fillMenuInfo(menu, Popup);
}

void  eraseAllData(QMenuData *data)
{
	QListT<QMenuItem>  	*items = ((MenuDataHelper*) data)->Items();
	QListT<QPopupMenu>	popups;

	if (items)
	for (items->first(); items->current(); items->next())
	if (items->current()->popup())
	{
		popups.append(items->current()->popup());
	}
	data->clear();
	for (popups.first(); popups.current(); popups.next())
	{
		eraseAllData(popups.current());
		delete popups.current();
	}
}

void  createMenuData(QMenuData *data, QMenuInfo *info)
{
	if (!data || !info) return;
	
	for (QMenuLabel *l= info->labels.first(); l; l = info->labels.next())
	if (l->text != "" || !l->pixmap.isNull())
	{
		QPopupMenu  *menu = l->popup ? new QPopupMenu : NULL;
		if (menu) 
		{
			createMenuData(menu, l->popup);
			if (l->pixmap.isNull())
				data->insertItem(l->text, menu);
			else
				data->insertItem(l->pixmap, l->text, menu);
		} else
		{
			if (l->pixmap.isNull())
			{
				if (l->text == "-") data->insertSeparator();
				else
					if (l->clicked != "") data->insertItem(l->text, editor->designer, QString("1") + l->clicked);
				else
					data->insertItem(l->text);
			}
			else
			{
				if (l->clicked != "")
					data->insertItem(l->pixmap, l->text, editor->designer, QString("1") + l->clicked);
				else
					data->insertItem(l->pixmap, l->text);
			}
		}
	}
}

void MenuEditor::closeEvent(QCloseEvent *e)
{
// transfer back menu data to QMenuBar	

	eraseAllData(menu); 
        // el menu esta ahora vacio
        // crear los sociales

	createMenuData(menu, menuData);

	if (menuData) delete menuData;

	InspectorDataList List;
	List.append(obj, &_QMenuBar);
	objectInspector->EditProperties(designer, obj, List);
	((MenuEditor*)designer->Widget())->clearWFlags(WState_Disabled);
	
	QMainWindow::closeEvent(e); 	
	return ;
}

void MenuEditor::goOnEnter()
{

}

void  PaintItem(QPainter &p, QMenuLabel *item)
{
	if (!item) return;
	
	QRect R(item->pos.x()+2, item->pos.y()+2, 
	        item->pos.width()-4, item->pos.height()-4);
	
	if ( QString(item->text) == "" && item->pixmap.isNull())
	{
		QBrush	B(QColor(128, 128, 128));
		p.fillRect(R, B);
	}

	if (item == active)
	{
		QBrush	B(QColor(0, 128, 192));
		p.fillRect(R, B);
	}
	
	if (QString(item->text) == "-")
	{
		p.drawLine(item->pos.x(), item->pos.y()+item->pos.height()/2, 
		           item->pos.x()+item->pos.width(), item->pos.y()+item->pos.height()/2);
	} else
	{
		// paint pixmap is any

		if (item->parent->popup || item->pixmap.isNull())
		p.drawText(item->pos.x() + item->parent->pixmaped + 4, 
		           item->pos.y() + p.fontMetrics().height(), item->text);
	}
} 


void MenuEditor::paintEvent(QPaintEvent *e)
{
   Enter("painEvent");


    QPainter p( this );
    QBrush	brush(QColor(192, 192, 192));
    if ( e )
	p.setClipRect( e->rect() );
    
   for (QMenuInfo *popup = popups.first(); popup; popup = popups.next())
   {
   	if (!popup->popup) popup->pos.setWidth(width());
   	
	p.fillRect(popup->pos, brush);
	p.drawRect(popup->pos);
		
	for (QMenuLabel *l=popup->labels.first(); l; l = popup->labels.next())
	{
		PaintItem(p, l);
	}
   }

   Leave("painEvent");
   
}




/********************************************************************/
void propsQMenuItem(ClassInfo *classinfo)
{
	new PropInfo(classinfo, &_string, "text", &QMenuLabel::getText, &QMenuLabel::setText);
	new PropInfo(classinfo, &_QPixmap, "pixmap", &QMenuLabel::getPixmap, &QMenuLabel::setPixmap);
	new PropInfo(classinfo, &_signal, "clicked()", &QMenuLabel::getClicked, &QMenuLabel::setClicked);
}
ClassInfo _QMenuLabel("QMenuItem", NULL, NULL, &propsQMenuItem);

void	MenuEditor::mousePressEvent(QMouseEvent *e)
{
	QMenuLabel 	*last = active;
	QMenuLabel	*item;


	Enter("mousePressEvent");

	if (e->button() == LeftButton && LabelAt(e->x(), e->y(), &item))
	{
	
		InspectorDataList  List;
		List.append(item, &_QMenuLabel);
		objectInspector->EditProperties(designer, item, List); 
		
		if (item != active)
		{
			active = item;

			if (last) repaint(last->pos, false);
			
			Activate(active->parent);
			repaint(active->parent->pos, false);

			if (active->popup)
			{
				Activate(active->popup);
				repaint(active->popup->pos, false);
			}
			repaint(active->pos, false);
		}
	}
	Leave("mousePressEvent");
}
	


void MenuEditor::checkLastItem(QMenuLabel *item)
{
   // si no es el ultimo del menu salir
   if (item != item->parent->labels.last()) return;
   
   // si es el ultimo de un popup poner otro y salir
   if (item->parent->popup)
   {
   	item->parent->labels.append(new QMenuLabel(item->parent));
   	ReArrangeMenu(*item->parent);
	repaint(true);
	return;
   }
   
   // si es el ultimo de un menubar ponerle un submenu y adicionar otro al final
   addPopupTo(item);
   item->parent->labels.append(new QMenuLabel(item->parent));
   ReArrangeMenu(*item->parent);
   repaint(true);
}


void MenuEditor::addPopupTo(QMenuLabel *item)
{
   if (!item->popup) 
   {
   	item->popup = new QMenuInfo(item);
   	ReArrangeMenu(*item->parent);
   }
}

void MenuEditor::keyPressEvent(QKeyEvent *e)
{
	if (!active) return;
	
	if (e->ascii() == 127) // delete
	{
		if (active->parent->labels.count() > 1) delete active;
		else 
		if ( active->parent != menuData ) delete active->parent;
		else
		{
			QMessageBox::information(this, "VisualQt", "You can not delete this item.", 0x101);
			return;
		} 

		active = NULL;
		ReArrangeMenu(*menuData);
		repaint(true);	
	}
}

