/****************************************************************************
    files.cpp  30/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "files.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <cstdlib>
#include <stdio.h>
#include <cerrno>


QString exePath;



QString  projdir()
{
	char homeDir[1024];

	
	strncpy(homeDir, getenv("HOME"), 1023);
	strncat(homeDir, "/projects", 1023);

	if (chdir(homeDir) != 0 && errno == ENOENT && mkdir(homeDir, 0750) != 0) 
	{
		printf("mkdir %s : cannot create", homeDir);
	}
	return QString(homeDir) + "/";
} 




bool     fileExist(const QString name)
{
	FILE  *f;
	if ((f=fopen(name, "r"))) fclose(f);
	return f != NULL;
}


bool saveASCIIFile(QString fileName, QString text, bool clear)
{
	FILE  *f = fopen(fileName, clear ? "w" : "a");
	if (f) 
	{
		fwrite(text, 1, text.length(), f);
		fclose(f);
		return TRUE;
	} 
	return FALSE;
}


QString loadASCIIFile(QString fileName)
{
	QString  r;
	
	FILE  *f = fopen(fileName, "r");
	if (f) 
	{
		char *tmp;
		long  size;
		fseek(f, 0, SEEK_END);
		size = ftell(f);
		tmp = (char*) malloc(size+1);
		fseek(f, 0, SEEK_SET);
		fread(tmp, 1, size, f);
		fclose(f);
		tmp[size] = 0;
		r = tmp;
		free(tmp);
	} 
	return r;
}

QString  getCurrentDir()
{
	char buffer[1024];
	QString  r = getcwd(buffer, 1024);
	return r;
}

QString  getFilePath(const char *dir)
{
	QString r = dir;
	char *p = rindex(dir, '/');
	
	if (p) r = r.left(p-dir);
	return r;
}

QString  getFileName(const char *dir)
{
	char *p = rindex(dir, '/');
	
	if (p) return p+1; 
	return dir;
}

QString  getFileExt(const char *dir)
{
	char *p = rindex(dir, '.');
	
	if (p) return p; 
	return "";
}

void fileSplit(const char *File, QString &Path, QString &Name, QString &Ext)
{
	char tmp[2048], *p;

	strcpy(tmp, File);

	p = rindex(tmp, '/');
	if (p) 
	{
		Name = p+1;
		p[1] = 0;
		Path = tmp;

		char 	tmp1[2048];
		getcwd(tmp1, 2047);
		chdir(tmp);
		getcwd(tmp, 2047);
		chdir(tmp1);

		Path = QString(tmp) + "/";

	} else
	{
		Name = tmp;
		Path = "";
	}

	strcpy(tmp, Name);

	p = rindex(tmp, '.');
	if (p) 
	{
		Ext = p;
		*p=0;
		Name = tmp;
	} else Ext = "";

}

void fileSplitRel(const char *File, const char *From, QString &Path, QString &Name, QString &Ext)
{
	char tmp[2048], *p;
	
	strcpy(tmp, From);
	
	Path = "";
	do
	{
		p = rindex(tmp, '/');
		if (p)
		{
			p[1] = 0;
			if (strncmp(tmp, File, strlen(tmp)) == 0)
			{
				Path = Path + &File[strlen(tmp)];
				strcpy(tmp, Path);
				File = tmp;
				break;
			}
			Path = Path + "../";
			p[0] = 0;
		}
	} while (p && (strlen(tmp)>0)); // no hay nada en comun

	fileSplit(File, Path, Name, Ext);
}


#include <qapp.h>
#include <qfiledialog.h>
extern class MainWindow *mainWindow;

QString lastSavePath;

QString	 getSaveFileName(QString Name, QString Ext)
{
	Name = QFileDialog::getSaveFileName(lastSavePath+Name, Ext, (QWidget*) mainWindow);
	if (Name.length()>0)
	{
		fileSplit(Name, lastSavePath, Name, Ext);
		Name = lastSavePath + Name + Ext;
	}
	return Name;
}



QString lastOpenPath;

QString	 getOpenFileName(QString Name, QString Ext)
{
	Name = QFileDialog::getOpenFileName(lastOpenPath+Name, Ext, (QWidget*) mainWindow);
	if (Name.length()>0)
	{
		fileSplit(Name, lastOpenPath, Name, Ext);
		Name = lastOpenPath + Name + Ext;
	}
	return Name;
}


void  fileInitialize(char *argv)
{
    QString t1(argv), t2;

    char cwd[1024];
    getcwd(cwd, 1024);	
   
    if ( *argv != '/' ) t1 = QString(cwd) + "/" + argv;
    
    fileSplit(t1, exePath, t1, t2); 

    lastSavePath = lastOpenPath = projdir();	

}
