/****************************************************************************
    designer.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "designer.h"
#include "../inspector/inspector.h"
#include "../inspector/dsgnintf.h"
#include "../mainwnd.h"

#include <qpainter.h>
#include <stdio.h>
#include <stdarg.h>


int	GRID = 8;
#define BW 5

FormDesigner  *lastDesigner;



void	PaintWidget(QWidget *w, int W, int H, bool allowResize)
{
	QPainter Painter(w);
	QBrush   Brush(Qt::SolidPattern);
	 			
	Painter.fillRect(0, 0, BW, BW, Brush);
	Painter.fillRect(W-BW, 0, BW, BW, Brush);
	Painter.fillRect(0, H-BW, BW, BW, Brush);
	Painter.fillRect(W-BW, H-BW, BW, BW, Brush);
	
	if (allowResize)
	{
		Painter.fillRect((W-BW) / 2, 0, BW, BW, Brush);
		Painter.fillRect(0, (H-BW) /2, BW, BW, Brush);
		Painter.fillRect(W-BW, (H-BW)/2, BW,BW, Brush);
		Painter.fillRect((W-BW)/2, H-BW, BW, BW, Brush);
	}
}

#define MOUSE(x) ((QMouseEvent*) x)


int 	FormDesigner::HitCode(Designer *D, QEvent *e)
{
	QWidget *C = D->Widget();

	if (C != (QWidget*) D->Instance()) return 0;
	
    	if (MOUSE(e)->y()<BW+1)
    	{
		if (MOUSE(e)->x()<BW+1)
		{
			C->setCursor(sizeFDiagCursor);
			return 1;
		} else
	    	if ( (MOUSE(e)->x()<(C->width()+BW+1)/2) && (MOUSE(e)->x()>(C->width()-BW-2)/2) )
	    	{
	    		C->setCursor(sizeVerCursor);
	    		return 2;
	    	} else
	    	if ( MOUSE(e)->x()>C->width()-BW-1 )
	    	{
	    		C->setCursor(sizeBDiagCursor);
	    		return 3;
	    	} else C->setCursor(arrowCursor);
    	} else
    	if ( (MOUSE(e)->y()<(C->height()+BW+1)/2) && (MOUSE(e)->y()>(C->height()-BW-3)/2) )
    	{
		if (MOUSE(e)->x()<BW+1)
		{
			C->setCursor(sizeHorCursor);
			return 4;
		} else
	    	if ( MOUSE(e)->x()>C->width()-BW-2 )
	    	{
	    		C->setCursor(sizeHorCursor);
	    		return 5;
	    	} else C->setCursor(arrowCursor);
    	} else
	if (MOUSE(e)->y()>C->height()-BW-2)
	{
		if (MOUSE(e)->x()<BW)
		{
			C->setCursor(sizeBDiagCursor);
			return 6;
		} else
	    	if ( (MOUSE(e)->x()<(C->width()+BW+1)/2) && (MOUSE(e)->x()>(C->width()-BW-3)/2) )
	    	{
	    		C->setCursor(sizeVerCursor);
	    		return 7;
	    	} else
	    	if ( MOUSE(e)->x()>C->width()-BW-2 )
	    	{
	    		C->setCursor(sizeFDiagCursor);
	    		return 8;
	    	} else C->setCursor(arrowCursor);
	} else C->setCursor(arrowCursor);	    	
    return 0;
}    

InterfaceList	*componentList = NULL;

ComponentClass::ComponentClass(char **pic, char *p, char *i, char *n, designerCreate c)
: palette(p), baseclass(0), classname(n), inc(i), img(pic), Create(c) 
{ 
	if (!componentList) componentList = new InterfaceList();
  	componentList->append(this);
}


ComponentClass* InterfaceList::operator[](const char *name)
{ 
	for ( first(); current(); next() )
	if (strcmp(current()->classname, name)==0 ||
	    (current()->baseclass && strcmp(current()->baseclass, name)==0) ) return current();
	return NULL;
}			

extern class ClassInfo _QMainWindow;

bool  FormDesigner::isDesignerEvent(Designer *D, QEvent *e)
{
    switch ( e->type() ) {
	case QEvent::User+0:
	case QEvent::User+1:
		return false;

	case 6: // QEvent::KeyPress: 

           if (D->designer->testWFlags(WState_Disabled)) return true;

	   if (((QKeyEvent*)e)->ascii() == 127) //  del
	   {
	        InspectorDataList  List;
	        
	        List.append(Instance(), &_QMainWindow);
		objectInspector->EditProperties(this, Instance(), List);

		selectionDelete();
           }
	   return TRUE;
	
	case QEvent::MouseMove:
           if (D->designer->testWFlags(WState_Disabled)) return true;
	    
	    if ( (D!=this) && (selection.count()==1) && (selection.find(D)!=-1) )
	    {
		HitCode(D, e);
	    }
	    return TRUE;
	    
	case QEvent::MouseButtonPress:
           if (D->designer->testWFlags(WState_Disabled)) return true;

	 if (MOUSE(e)->button() == LeftButton) 
	 {    
            if (MOUSE(e)->state() & ShiftButton) grid = 1;
            else grid = GRID;

	    // if Palette is clicked addComp
	    if ( 
	    	mainWindow->lastTool &&
	    	(D->Flags & cfNoChild) == 0 &&
	   	selectionCreate(D, MOUSE(e)->x(), MOUSE(e)->y(), mainWindow->lastTool->name())
               ) 
                    return TRUE; 
	
	    // is not control clicked on selection, select it		 
	    if ((D == this) || (selection.find(D)==-1)) 
	        selectionAppend(D, (MOUSE(e)->state() & ShiftButton)==0); 
	
	    // is it on selection, drag or size it
	    else 
	    {
	    	int code;
		if ( (selection.count()==1) && 
		     (code=HitCode(D, e)) &&
		     (D->Flags & cfNoSize) == 0
		   ) selectionSize(D, code);
	        else 
	        {
	        	if ( (D->Flags & cfNoMove) == 0 ) selectionDrag(D);
	        }
	    }
	 } else
	 if (MOUSE(e)->button() == RightButton)
	 {
	 }   
	 return TRUE;

	case QEvent::MouseButtonDblClick:
           if (D->designer->testWFlags(WState_Disabled)) return true;

	 if (MOUSE(e)->button() == LeftButton && selection.count()==1) 
	 {
		PropertyEditorClass  Editor = GetClassEditor(D->Instance()->className());
		if (Editor)
		{
			TPropertyEditor *editor = Editor->Create(NULL);
			editor->Edit();
			delete editor;
		 }
	 

	 
	 }
	 return true;
	    
	case QEvent::Paint:
	    D->defaultHandler(e);
	    
	    if (D==this) 
	    {	QPainter Painter( Widget() );
		for (int y=0;y<Widget()->height()+GRID;y+=GRID)
		for (int x=0;x<Widget()->width()+GRID;x+=GRID)
		{
			Painter.drawPoint(x, y);
		}
		return TRUE;
	    }
	    
	    if (selection.find(D)!=-1) 
	      PaintWidget(D->Widget(), D->Widget()->width(), D->Widget()->height(), selection.count()==1);

	    return TRUE;

	case QEvent::Move:
	case QEvent::Resize:
	    D->defaultHandler(e);
	    objectInspector->Update();
	    return TRUE;
	
	case QEvent::Leave:
		D->Widget()->setCursor( arrowCursor );
		return TRUE;

	default: 
	    return true;
    }
    return true;
}


extern "C" FormDesigner *activeDesigner();
FormDesigner *activeDesigner()
{
	return lastDesigner;
}