/****************************************************************************
    designer.h  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/
#ifndef _designer_
#define _designer_

#include <qapplication.h>
#include <qmainwindow.h>
#include <X11/Xlib.h>
#include <qtrtti.h>
#include <qmetaobject.h>
#include <stdio.h>

class FormDesigner;
class Designer;
class ComponentClass;

#define		cfNoSize	1
#define		cfNoMove	2
#define		cfNoChild	4
#define		cfNoShow	8

class InterfaceList: public QListT<ComponentClass> {
public:
	ComponentClass* operator[](const char *name);
};

extern InterfaceList  *componentList;

typedef Designer*  (*designerCreate)(Designer*);
class ComponentClass {
public:
	char		*palette;
	char		*baseclass;
	char		*classname;
	char		*inc;
	char 		**img;
	designerCreate 	Create;
	ComponentClass(char **pic, char *p, char *i, char *n, designerCreate c);
}; 

class  FormDesigner;

class  Designer {
private:
	Designer() { }
public:
	FormDesigner	*designer;
	int	Flags;
	virtual QObject *Instance() = 0;
	virtual QWidget *Widget() = 0;
public:
	Designer(Designer *p, int F): designer( p ? p->designer : NULL ), Flags(F) { }  
	virtual bool	defaultHandler(QEvent*) { return TRUE; }
	virtual bool	isDesignerEvent(Designer *, QEvent*) { return FALSE; }
	virtual ~Designer() { printf("se fue el designer\n"); }
	virtual void Destroy() { delete (Designer*) this; }
};


template<class Type> class Component: public Designer, public Type {
public:
	virtual QObject   *Instance() { return this; }
	virtual QWidget   *Widget() { return this; }      
public:
	Component(Designer *p, int F)
	: Designer(p, F), Type(p ? p->Widget() : NULL)
	{ 
		initMetaObject();
	}
	virtual bool	defaultHandler(QEvent *e) { return Type::event(e); }
	virtual bool 	event(QEvent *e) { return designer->isDesignerEvent(this, e) || Type::event(e); }
	virtual void    Destroy() { delete (Component<Type>*) this; }  
};

#define RegisterComponent(Img, Palette, Include, Type, F)	\
Designer *designerCreate##Type(Designer *p) \
{ return new Component<Type>(p, F); }	\
static ComponentClass class##Type(Img, Palette, Include, #Type, designerCreate##Type);


class  FormDesigner: public Designer, public QMainWindow {
public:									      
    QMetaObject *metaObject() const { return metaObj; }			      
    const char  *className()  const;					      
protected:								      
    	void	initMetaObject();
	QList<Designer>		Components;
private:								      
    QMetaData   *slot_tbl;
    QMetaObject *metaObj;
public:
	int			grid;
	QListT<Designer>	selection;

	Designer	*Find(QString);
	Designer	*Find(QObject*);
	void		AddReference(Designer *);
	void		RemoveReference(QObject *);
	
	void    selectionAppend(Designer *, bool clear);
	void    selectionRemove(Designer *);
	void	selectionDraw(Window Desktop, GC  gc, int DX, int DY);
	int	HitCode(Designer *, QEvent *);
	void	selectionDrag(Designer *);
	void	selectionSize(Designer *, int);
	bool    selectionCreate(Designer *, int X, int Y, const char *classname);
	void	selectionDelete();
public:
	virtual QObject   *Instance() { return this; }
	virtual QWidget   *Widget() { return this; }      
	void	doNothing();
	void    NonQObjectPropertyChanged();
	void    WriteObjectRef(FILE *f, QObject *Obj);
public:
	FormDesigner(QWidget *);
	virtual	~FormDesigner();
	virtual bool	isDesignerEvent(Designer *, QEvent*) ;
	virtual bool	defaultHandler(QEvent *e) { return QMainWindow::event(e); }
	virtual bool 	event(QEvent *e) 
	{ return designer->isDesignerEvent(this, e) || QMainWindow::event(e); }
	bool queryClose();
	void SaveRes(QString, QString);
	bool LoadRes(QString, QString);
	void CreateSlot(QString);
	void RenameSlot(QString, QString);
	void RenameComponent(QString, QString);
};

extern "C" void PaintWidget(QWidget *w, int W, int H, bool allowResize);

#endif


