/****************************************************************************
    formdsgn.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include <stdio.h>

#include "designer.h"
#include "../inspector/inspector.h"
#include "../mainwnd.h"
#include <qmetaobject.h>
#include "vtkreader.h"
#include <qdict.h>
#include <stdlib.h>
#include <qtrtti.h>

#define  MIN(x,y) ((x)<(y) ? (x) : (y))
#define  MAX(x,y) ((x)>(y) ? (x) : (y))


#define DESIGNER(D) ((Designer*) &((char*)(D))[-sizeof(Designer)])


/*
  Calculate optimal dictionary size for n entries using prime numbers,
  and assuming there are no more than 40 entries.
*/

class QMemberDict : public QAsciiDict<QMetaData>
{
public:
    QMemberDict(int size=17,bool cs=TRUE,bool ck=TRUE) :
	QAsciiDict<QMetaData>(size,cs,ck) {}
    QMemberDict( const QMemberDict &dict ) : QAsciiDict<QMetaData>(dict) {}
    ~QMemberDict() { clear(); }
    QMemberDict &operator=(const QMemberDict &dict)
    { return (QMemberDict&)QAsciiDict<QMetaData>::operator=(dict); }
};


class TMetaObject				// meta object class
{
public:
    char	*classname;			// class name
    char	*superclassname;		// super class name
    QMetaObject *superclass;			// super class meta object
    void *p1;
    void *p2;
    QMetaData	*slotData;			// slot meta data
    QMemberDict *slotDict;			// slot dictionary
    QMetaData	*signalData;			// signal meta data
    QMemberDict *signalDict;			// signal dictionary
};


QString FindName(Designer *Obj)
{
	for (int i=1; i<10000; i++)
	{
		char s[256];
		sprintf(s, "%s%d", Obj->Instance()->className(), i);
		if (Obj == Obj->designer)
		{
			if (!mainWindow->findForm(s)) return s;		
		}
		else
		if (FindChild(Obj->designer->Instance(), s)==NULL)
		{
			return s;
		}
	} 
	return "noname";
}


ComponentClass IQMainWindow(NULL, "Qt", "", "QMainWindow", NULL);


void  FormDesigner::doNothing()
{

}

FormDesigner::FormDesigner(QWidget *)
: Designer(NULL, 0), QMainWindow(NULL)
{
    QMainWindow::initMetaObject();
    slot_tbl = new QMetaData[256];
    memset(slot_tbl, 0, 256*sizeof(QMetaData));
    metaObj = new QMetaObject("QFormDesigner", "QMainWindow",
	0, 0,
	0, 0 );
    ((TMetaObject*)metaObj)->slotData = slot_tbl;
    ((TMetaObject*)metaObj)->slotDict = new QMemberDict(17, TRUE, FALSE);

    designer = this;

    setName("FormDesigner");
    setCaption(FindName(this));
    setGeometry(300, 150, 400, 200);


    Components.setAutoDelete(false);
    selection.setAutoDelete(false);
}

FormDesigner::~FormDesigner()
{
	InspectorDataList  List;
	objectInspector->EditProperties(NULL, NULL, List);
}

Designer *FormDesigner::Find(QString Name)
{
	if ( Name.length() > 0 )
	{
		if ( Name == name() ) return this;
		for (Designer *l = Components.first(); l; l = Components.next())
		if ( Name == l->Instance()->name() ) return l;
	}
	return NULL;
}

Designer *FormDesigner::Find(QObject *Obj)
{
	if ( Obj != NULL )
	{
		if ( Obj == this ) return this;
		for (Designer *l = Components.first(); l; l = Components.next())
		if ( Obj == l->Instance() ) return l;
	}
	return NULL;
}

void FormDesigner::selectionAppend(Designer *item, bool clear)
{
	clear |= (item && selection.first() && 
	(item->Widget()->parentWidget()!=selection.current()->Widget()->parentWidget())) ||
	(item && item->Widget()->inherits("QMainWindow"));

	if (clear) 
	while ( Designer *tmp = selection.first() )
	{
		tmp->Widget()->setMouseTracking(FALSE);
		tmp->Widget()->setCursor( arrowCursor );
		selection.remove(tmp);
		tmp->Widget()->repaint(TRUE);
	}
	if (item)
	{ 
		item->Widget()->setMouseTracking(item != this);
		if (selection.find(item)==-1) selection.append(item);
		selection.first()->Widget()->repaint(TRUE);
		item->Widget()->repaint(FALSE);
	}
	
	InspectorDataList  List;
	for ( selection.first(); selection.current(); selection.next() )
		List.append( selection.current()->Instance(), ClassInfo::Ancestor(selection.current()->Instance() ));
	objectInspector->EditProperties(this, Instance(), List);
}


void  FormDesigner::AddReference(Designer *item)
{
	if ( item ) Components.append( item );
}

void  FormDesigner::RemoveReference(QObject *item)
{
	// get the object designer 
	Designer *tmp = Find( item );

	if ( !tmp ) return;
	
	// get object children list
	QList<QObject> *l = (QList<QObject>*) item->children();

	if (l)
	for (l->first(); l->current(); l->next() )
	{
		RemoveReference( l->current() );
	}
	
	// erase reference from source
	mainWindow->RemoveComponent(this, tmp);

	// remove reference
	Components.remove( tmp );
}

void FormDesigner::selectionDelete()
{
	while ( selection.first() && (selection.first()!=this) )
	{
		Designer *tmp = selection.first();
		if ( !Find(tmp->Instance()) ) // error porque como es posible una seleccion de algo que no este
			return;
		selection.remove(tmp);
		RemoveReference( tmp->Instance() );
		tmp->Destroy();
	}
}

void FormDesigner::selectionRemove(Designer *item)
{
	selection.remove(item);
	item->Widget()->repaint(FALSE);
}

#define  dpy  ((_XDisplay*) qt_xdisplay())

void  FormDesigner::selectionDraw(Window Desktop, GC  gc, int DX, int DY)
{
	int		X, Y, W, H;

	for( selection.first(); selection.current(); selection.next() )
	{
		X = selection.current()->Widget()->x() + DX;
		Y = selection.current()->Widget()->y() + DY;
		W = selection.current()->Widget()->width();
		H = selection.current()->Widget()->height();
		XDrawRectangle(dpy, Desktop, gc, X, Y, W, H);
	}
}
	


void	FormDesigner::selectionDrag(Designer *D)
{
	XGCValues	v;
	GC		gc;
	Window 		w;
	int		W, H, X, Y, X1, Y1, DX, DY, x, y;
	unsigned int 	m;
	XEvent		e;
	Window		Desktop;
		
	if (!D || D->Widget()->inherits("QMainWindow") || !D->Widget()->parent()) return;
	
	Desktop = D->Widget()->parentWidget()->winId();

	
	v.function = GXxor;
	v.line_width = 0;
	v.foreground = 0xffff;
	v.subwindow_mode = IncludeInferiors;
	gc = XCreateGC(dpy, Desktop, GCFunction|GCLineWidth|GCForeground|GCSubwindowMode, &v);	
	
	
	XQueryPointer(dpy, Desktop, &w, &w, &X, &Y, &x, &y, &m);
	
	X = (X / grid) * grid;
	Y = (Y / grid) * grid;
	DX = DY = 0;
	selectionDraw(Desktop, gc, DX, DY);
	XGrabPointer(dpy, Desktop, 1, ButtonReleaseMask, GrabModeAsync, GrabModeAsync, Desktop, 0, 0);
	for (;;) {
		if (XCheckMaskEvent(dpy, 
			ButtonPressMask | ButtonReleaseMask |
			EnterWindowMask | LeaveWindowMask |
			PointerMotionMask, &e) &&
		   (e.type==ButtonRelease) &&
		   (e.xbutton.button==1)) break;
		XQueryPointer(dpy, Desktop, &w, &w, &X1, &Y1, &x, &y, &m);
		X1 = (X1 / grid) * grid;
		Y1 = (Y1 / grid) * grid;
		if ((X+DX!=X1) || (Y+DY!=Y1)) {
			selectionDraw(Desktop, gc, DX, DY);
			DX = X1 - X;
			DY = Y1 - Y;
			selectionDraw(Desktop, gc, DX, DY);
		}
	}
	XUngrabPointer(dpy, 0);
	selectionDraw(Desktop, gc, DX, DY);
	
	XFreeGC(dpy, gc);


	if (DX || DY)
	for (selection.first(); selection.current(); selection.next())
	{
		X = selection.current()->Widget()->x() + DX;
		Y = selection.current()->Widget()->y() + DY;
		W = selection.current()->Widget()->width();
		H = selection.current()->Widget()->height();
		selection.current()->Widget()->setGeometry(X, Y, W, H);
	}
}

void	FormDesigner::selectionSize(Designer *D, int code)
{
	XGCValues	v;
	GC		gc;
	Window 		w;
	int		W, H, X, Y, X1, Y1, XP, YP, x, y;
	unsigned int 	m;
	XEvent		e;
	Window		Desktop;
		
	if (!D || D->Widget()->inherits("QMainWindow") || !D->Widget()->parent()) return;
	
	Desktop = D->Widget()->parentWidget()->winId();

	
	v.function = GXxor;
	v.line_width = 0;
	v.foreground = 0xffff;
	v.subwindow_mode = IncludeInferiors;
	gc = XCreateGC(dpy, Desktop, GCFunction|GCLineWidth|GCForeground|GCSubwindowMode, &v);	
	
	
	XQueryPointer(dpy, Desktop, &w, &w, &X1, &Y1, &x, &y, &m);
	
	X = selection.current()->Widget()->x();
	Y = selection.current()->Widget()->y();
	W = X + selection.current()->Widget()->width() - 1;
	H = Y + selection.current()->Widget()->height() - 1;
	XDrawRectangle(dpy, Desktop, gc, X, Y, 1+W-X, 1+H-Y);

	XGrabPointer(dpy, Desktop, 1, ButtonReleaseMask, GrabModeAsync, GrabModeAsync, Desktop, 0, 0);
	for (;;) {
		if (XCheckMaskEvent(dpy, 
			ButtonPressMask | ButtonReleaseMask |
			EnterWindowMask | LeaveWindowMask |
			PointerMotionMask, &e) &&
		   (e.type==ButtonRelease) &&
		   (e.xbutton.button==1)) break;
		XQueryPointer(dpy, Desktop, &w, &w, &XP, &YP, &x, &y, &m);
		if ((XP!=X1) || (YP!=Y1)) {
			XDrawRectangle(dpy, Desktop, gc, X, Y, 1+W-X, 1+H-Y);
			switch ( code ) {
			case 1:
				X = MIN(W, X+XP-X1);			
				Y = MIN(H, Y+YP-Y1);
				break;
			case 2:
				Y = MIN(H, Y+YP-Y1);
				break;
			case 3:
				W = MAX(X, W+XP-X1);			
				Y = MIN(H, Y+YP-Y1);
				break;
			case 4:
				X = MIN(W, X+XP-X1);			
				break;
			case 5:
				W = MAX(X, W+XP-X1);			
				break;
			case 6:
				X = MIN(W, X+XP-X1);			
				H = MAX(Y, H+YP-Y1);
				break;
			case 7:
				H = MAX(Y, H+YP-Y1);
				break;
			case 8:
				W = MAX(X, W+XP-X1);			
				H = MAX(Y, H+YP-Y1);
				break;
			}
			X1 = XP;
			Y1 = YP;
			XDrawRectangle(dpy, Desktop, gc, X, Y, 1+W-X, 1+H-Y);
		}
	}
	XUngrabPointer(dpy, 0);
	XDrawRectangle(dpy, Desktop, gc, X, Y, 1+W-X, 1+H-Y);
	
	XFreeGC(dpy, gc);


	for (selection.first(); selection.current(); selection.next())
		selection.current()->Widget()->setGeometry(X, Y, 1+W-X, 1+H-Y);
}


bool    FormDesigner::selectionCreate(Designer *Parent, int X, int Y, const char *classname)
{
	ComponentClass *Class = (*componentList)[classname];
	if (!Class || !Class->Create) return FALSE;

	Designer *child = Class->Create(Parent);
	if ( Class->baseclass == NULL ) Class->baseclass = strdup( child->Instance()->className() );

        if ((child->Flags & cfNoMove) == 0) 
		child->Widget()->move((X / grid)*grid, (Y / grid)*grid);

        child->Widget()->show();

        child->Instance()->setName(FindName(child));
	selectionAppend(child, TRUE);
	AddReference(child);
	
	mainWindow->designerAddComp(this, Parent, child);

	return TRUE;
}

bool FormDesigner::queryClose()
{
	hide();
	return FALSE;
}





// *************************************************************************


#include <qmetaobject.h>


const char *FormDesigner::className() const
{
    return "QMainWindow";
}

void FormDesigner::initMetaObject()
{
    return  ;
}


void FormDesigner::CreateSlot(QString Name)
{
    typedef void(FormDesigner::*m1_t0)();
    m1_t0 v1_0 = &FormDesigner::doNothing;
    int i;


    for (i=0; i<256; i++)
    if (!slot_tbl[i].name || !*slot_tbl[i].name)
    {
	slot_tbl[i].name = strdup(Name);
	slot_tbl[i].ptr = *((QMember*)&v1_0);
	((TMetaObject*) metaObj)->slotDict->insert(slot_tbl[i].name, &slot_tbl[i]);

	mainWindow->CreateSlot(this, Name);

	break;
    } 
 
}

void FormDesigner::RenameSlot(QString Name, QString newName)
{
    typedef void(FormDesigner::*m1_t0)();
    m1_t0 v1_0 = &FormDesigner::doNothing;

    for (int i=0; i<256; i++)
    if (slot_tbl[i].name && (strcmp(slot_tbl[i].name, Name)==0) )
    {
	((TMetaObject*) metaObj)->slotDict->remove(slot_tbl[i].name);
	free( (void*) slot_tbl[i].name);
	slot_tbl[i].name = strdup(newName);
	slot_tbl[i].ptr = *((QMember*)&v1_0);
	((TMetaObject*) metaObj)->slotDict->insert(slot_tbl[i].name, &slot_tbl[i]);

	mainWindow->RenameSlot(this, Name, newName);

	break;
    } 

}


void  FormDesigner::WriteObjectRef(FILE *f, QObject *Obj)
{
	QListT<QObject> *l = (QListT<QObject>*) Obj->children();
	if (l)
	for (l->first(); l->current(); l->next())
	if ( Find( l->current() ) )
	{
		fprintf(f, "    %s = (%s*) FindChild(this, \x22%s\x22);\n", 
		    l->current()->name(), l->current()->className(), l->current()->name()); 
		WriteObjectRef(f, l->current());
	}
}

class TVQTWriter: public TWriter
{
	FormDesigner	*designer;
public:
	TVQTWriter(const char *c, FormDesigner *f): TWriter(c), designer(f) {}
	bool stored(QObject *o) 
	{ 
		return TWriter::stored(o) && designer->Find( o ) != NULL; 
	}

};

void  FormDesigner::SaveRes(QString Path, QString Name)
{

	TVQTWriter  writer(Path+Name+".vqt", this);
	writer.writeObject( Instance() );
	
	FILE  *f = fopen(Path+Name+".c++", "w");
	FILE  *f1 = fopen(Path+Name+".vqt", "a+");
	int   size = ftell(f1);
	fseek(f1, 0, SEEK_SET);
	
	fprintf(f, "#include \x22%s.h\x22\n#include <reader.h>\n", Name.data());
	
	fprintf(f, "static unsigned char rc%s[] = {\n%d, %d, %d, %d,\n", 
	 name(),
	 size & 255,
	 size >> 8 & 255,
	 size >> 16 & 255, 
	 size >> 24 & 255);
	for (int j=0; j<size; j++)
	{
		int c = 0;
		fread(&c, 1, 1, f1);
		if (!((j + 1) % 32)) fprintf(f, "\n");
		fprintf(f, "%d,", c);
	}
	fprintf(f, "\n};\n\n");	

	fprintf(f, "void %s::initRCData()\n{\n    ReadFormResource((unsigned int *) &rc%s, this);\n", 
		name(), name());

	WriteObjectRef(f, this);

	fprintf(f, "}\n");

	fclose(f);
}

bool  FormDesigner::LoadRes(QString Path, QString Name)
{
	VReader::ReadFormTemplate(Path+Name+".vqt", this );

	return true;
}

void  FormDesigner::RenameComponent(QString Name, QString newName)
{
	QObject *Child = FindChild(this, Name);
	
	if (selection.first() && selection.first()->Instance() == Child)
	{
		mainWindow->RenameComponent(this, selection.first(), newName);
	}
}

void  FormDesigner::NonQObjectPropertyChanged()
{
	for( selection.first(); selection.current(); selection.next() )
        selection.current()->Widget()->repaint(true);
}