/****************************************************************************
    dsgnedit.h  23/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "inspector.h"

#include <qlineedit.h> 
#include <qpushbutton.h> 
#include <qcombobox.h> 


class TPropEdit {
protected:
	TPropertyEditor	*Property;
public:
	int	modified;
	virtual int	Modified() { return modified; }
	TPropEdit(TPropertyEditor *p): Property(p), modified(0) { }
	virtual ~TPropEdit() { delete Widget(); }
	virtual QWidget *Widget() { return NULL; }
	virtual	void Show() {}
	virtual void Update() {}	
	virtual void Sync() {}
	virtual void Edit() { if (Property) Property->Edit(); }
};


class TLineEdit: public TPropEdit, QLineEdit {
public:
	TLineEdit(QWidget *Owner, TPropertyEditor *p): TPropEdit(p), QLineEdit(Owner) 
	{ 
		Update();
		connect(this, SIGNAL(returnPressed()), Owner, SLOT(OnClickSlot()));
		connect(this, SIGNAL(textChanged(const QString&)), Owner, SLOT(OnTextSlot(const QString&)));
	}
	virtual QWidget *Widget() { return this; }
	virtual void Update() { modified = 0; setText(Property->GetValue()); }	
	virtual void Sync() { modified = 0; Property->SetValue(text()); }	
};

class TListEdit: public TPropEdit, QComboBox {
public:
	TListEdit(QWidget *Owner, TPropertyEditor *p): TPropEdit(p), QComboBox(TRUE, Owner) 
	{ 
		connect(this, SIGNAL(activated(const QString&)), Owner, SLOT(OnListSlot(const QString&))); 
		setInsertionPolicy(QComboBox::NoInsertion);
	}
	int	Modified() { return Property->GetValue() != currentText(); }
	virtual QWidget *Widget() { return this; }
	virtual void Update();
	virtual void Sync();
};

class TDialogEdit: public TPropEdit, QPushButton {
public:
	TDialogEdit(QWidget *Owner, TPropertyEditor *p): TPropEdit(p), QPushButton(Owner) 
	{ 
	  	setText("...");
	  	connect(this, SIGNAL(clicked()), Owner, SLOT(OnEditSlot()));
	}
	virtual QWidget *Widget() { return this; }
};


