/****************************************************************************
    dsgnintf.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/


#include "inspector.h"
#include "dsgnintf.h"
#include "dsgnedit.h"
#include <qmetaobject.h>
#include "../formedit/designer.h"
#include <qobjectdict.h>

PropertyEditorMapper *mapperList = NULL;



PropertyEditorMapper::PropertyEditorMapper(const char *c, const char *p, void *e)
: Class(c), Prop(p), editor(e)
{
	printf("Property editor %s->%s registered\n", c, p);
	next = mapperList;
	mapperList = this;
}


int	TPropertyEditor::GetOrdValue()
{ int Value = 0;
  for (uint i=0; i<props.count(); i++) Value += GetOrdValueAt(i);
  return (props.count() ? Value / props.count() : 0);
}   

void	TPropertyEditor::SetOrdValue(int Value)
{ 
	for (uint i=0; i<props.count(); i++) 
	{
		SetOrdValueAt(i, Value); 
	}
	if (!aProp(0)->owner->inherits("QObject")) 
	  	objectInspector->designer->NonQObjectPropertyChanged();
}

QString TPropertyEditor::GetStrValue()
{ 
	return (props.count() && AllEquals()) ? GetStrValueAt(0) : QString(""); 
}

void	TPropertyEditor::SetStrValue(const char *Value)
{ 
	for (uint i=0; i<props.count(); i++) 	SetStrValueAt(i, Value); 
	if (!aProp(0)->owner->inherits("QObject")) 
	  	objectInspector->designer->NonQObjectPropertyChanged();
}


void TSetProperty::GetProperties(TPropertyEditorList &Editors)
{
	PropInfo *p = aProp(0);
	
	if ( p->type->kind != tkset ) return;

	EnumInfo *e = ((SetInfo*) p->type)->base;
	
	for ( EnumPair *pair = e->names; pair; pair = pair->next )
	 {
		TPropertyEditor *Editor = new TSetItemProperty(this, pair->name);

		for ( props.first(); props.current(); props.next() )
		{
			Editor->props.append( props.current()->instance, props.current()->propinfo );
		}
		Editors.inSort(Editor);
	 }
}


TSetItemProperty::TSetItemProperty(TSetProperty *s, const char *nm)
: TEnumProperty(), set(s), Name(nm)
{
	Index = s ? s->Index + 1 : 1;
}

QString TSetItemProperty::GetStrValueAt(int i)
{
	PropInfo *p = aProp(i);
	
	if ( p->type->kind != tkset ) return "false";

	EnumInfo *e = ((SetInfo*) p->type)->base;
	
	for ( EnumPair *pair = e->names; pair; pair = pair->next )
	if ( Name == pair->name )
	{
		int  Value = getOrdProp(aInst(i), aProp(i));
		
		if ( (Value & pair->value)==pair->value ) return "true";
		return "false";
	}
	return "false";
}

void TSetItemProperty::SetStrValueAt(int i, const char *value)
{
	PropInfo *p = aProp(i);
	
	if ( p->type->kind != tkset ) return;

	EnumInfo *e = ((SetInfo*) p->type)->base;
	
	for ( EnumPair *pair = e->names; pair; pair = pair->next )
	if ( Name == pair->name )
	{
		int  Value = getOrdProp(aInst(i), aProp(i)) & ~pair->value;
		if ( QString(value) == "true" ) Value |= pair->value;
		
		setOrdProp(aInst(i), aProp(i), Value);
		return;
	}
}

void TSetItemProperty::GetValues(QStrList &Values)
{
	Values.append("true");
	Values.append("false");
}

void TClassProperty::GetProperties(TPropertyEditorList &Editors)
{
	InspectorDataList objs;
	
	for (uint i=0; i<props.count(); i++)
	if (aProp(i)->type->kind == tkclass)
	{
		objs.append((void*) GetOrdValueAt(i), (ClassInfo*) aProp(i)->type);
	}

	GetComponentProperties(this, objs, Editors);
}



PropEditorClass<TOrdinalProperty> _TOrdinalProperty("Ordinal");
PropEditorClass<TStringProperty> _TStringProperty("String");
PropEditorClass<TClassProperty> _TClassProperty("Class");
PropEditorClass<TEnumProperty> _TEnumProperty("Enum");
PropEditorClass<TSetProperty> _TSetProperty("Set");


void * PropClassMap[] = 
{ 
  NULL, 		// tkundef
  &_TOrdinalProperty,   // tkord
  &_TStringProperty,    // tkstring
  &_TStringProperty,    // tkqstring
  &_TEnumProperty,	// tkenum
  &_TSetProperty,	// tkset
  &_TClassProperty,  	// tkclass
  NULL,			// tksignal
};



PropertyEditorMapper *GetPropertyEditor(ClassInfo *Class, const char *Prop)
{
	for (;Class;Class=Class->ancestor)
	for(PropertyEditorMapper *m = mapperList; m; m = m->next)
	if ( (strcmp(Class->name, m->Class)==0) && (strcmp(Prop, m->Prop) == 0) ) return m;
	
	return NULL;
}

PropertyEditorMapper *GetClassEditor(ClassInfo *Class)
{
	for (;Class;Class = Class->ancestor)
	for(PropertyEditorMapper *m = mapperList; m; m = m->next)
	{
		if ( 
		     !strcmp(Class->name, m->Class) && 
		     (!m->Prop || !*m->Prop) 
		    ) return m;
	}
	return NULL;
}


PropertyEditorClass GetClassEditor(const char *Class)
{
	// buscar si hay algun editor registrado para esa clase
	PropertyEditorMapper *m = GetClassEditor(ClassInfo::Find(Class));

	return (PropertyEditorClass) (m ? m->editor : NULL);
}



PropertyEditorClass GetEditorClass(void *o, ClassInfo *classinfo, PropInfo *propinfo)
{
	// buscar si hay algun editor registrado para esa propiedad
	PropertyEditorMapper *m = GetPropertyEditor(classinfo, propinfo->name);
	
	if (m) return (PropertyEditorClass) m->editor;
	
	// si la propiedad es de tipo class y hay un editor registrado
	if ( propinfo->type->kind == tkclass) 
	   	m = GetClassEditor((ClassInfo*) propinfo->type);

	if (m) return (PropertyEditorClass) m->editor;
	

	return (PropertyEditorClass) PropClassMap[propinfo->type->kind];
}





bool GetProperties(ClassInfo *classinfo, QListT<PropInfo> &props)
{
	if (!classinfo) return FALSE;
	
	for ( ; classinfo; classinfo=classinfo->ancestor)
	for (PropInfo *propinfo=classinfo->props; propinfo; propinfo=propinfo->next)
	if (propinfo->type->kind != tksignal)
	{
		props.append(propinfo);
	}
	return props.count() > 0;
}


void GetComponentProperties(TPropertyEditor *Parent, InspectorDataList &objs, TPropertyEditorList &Editors)
{ 
 
  QListT<PropInfo> 	Candidates;
  TPropertyEditor 	*Editor;
  PropertyEditorClass 	EdClass; 
  ClassInfo		*classinfo;
  PropInfo		*propinfo;

  if (!objs.count()) return;

  //  tomar todas las propiedades del primer objeto
  GetProperties(objs.cls(0), Candidates);
  
  // eliminar todas aquellas que no puedan ser editadas o
  // que no existan en todos los objetos o
  // que no sean del mismo tipo


  for (objs.first(); objs.current(); objs.next() )
  {
     classinfo = objs.current()->cls;
     for (int p=Candidates.count(); p; p-- )
     {
	// propiedad que vamos a revisar
     	PropInfo *tmp = Candidates.at(p-1);
     	
     	// esta en el componente p
     	PropInfo *tmp1 = classinfo->Prop(tmp->name);
 
	if (!tmp1 ||                                            // comp p no la tiene  
	   (tmp->type->kind != tmp1->type->kind) ||             // no son del mismo tipo
	   (!(EdClass = GetEditorClass(objs.current()->obj, classinfo, tmp1)))    // no tiene editor
	   
	   ) Candidates.remove(p-1);   else          // eliminar

	if ( objs.count()>1 )
	   {
	   	// Si el editor no acepta multi edicion tambien eliminar
	   	
	   	Editor = EdClass->Create(Parent);
		if ((Editor->GetAttributes() & paMultiSelect) == 0)
			Candidates.remove(p-1);
		delete Editor;
	   } 
     }
  }


  // ahora creamos los editores.
  for (Candidates.first(); Candidates.current(); Candidates.next())
  {
        // primero se crea el editor para la propiedad utilizando cualquiera de
        // los objetos (la propiedad tiene el mismo editor en todos los objetos
	propinfo = Candidates.current();
	
	EdClass = GetEditorClass(objs.obj(0), objs.cls(0), Candidates.current());
	Editor = EdClass->Create(Parent);
	  	
  	// ahora adicionamos todos los pares de componente::propiedad
	// a dicho editor
	for (objs.first(); objs.current(); objs.next() )
	{
  		Editor->props.append( objs.current()->obj, objs.current()->cls->Prop(propinfo->name) );
  	}
  	Editors.inSort(Editor);
  }  	
}


void GetSignals(QStrList &Signals, void *o, ClassInfo *cls)
{	
	if ( !o || !cls) return;

	if ( cls->inherits("QObject") ) getSignals((QObject*) o, Signals);

	for (; cls; cls=cls->ancestor)
	for (PropInfo *tmp = cls->props; tmp; tmp = tmp->next)
	if (tmp->type->kind == tksignal)
	  	Signals.append(tmp->name);
}



void GetComponentSignals(TPropertyEditor *Parent, InspectorDataList &objs, TPropertyEditorList &Editors)
{
 
  QStrList		Candidates;
  TPropertyEditor 	*Editor;


  if (!objs.count()) return;

  //  tomar todas las propiedades del primer objeto
  GetSignals( Candidates, objs.obj(0), objs.cls(0) );
  
  // eliminar todas aquellas que no puedan ser editadas o
  // que no existan en todos los objetos o
  // que no sean del mismo tipo

  for (objs.first(); objs.current(); objs.next() )
  {
     QStrList	check;
     GetSignals( check, objs.current()->obj, objs.current()->cls );
     for (int p=Candidates.count(); p; p-- )
     {
	// propiedad que vamos a revisar
     	char* tmp = Candidates.at(p-1);
     	
     	// esta en el componente p
     	for ( check.first(); check.current(); check.next() )
     	if (strcmp(check.current(), tmp)==0)
     	{
     		tmp = NULL;
     		break;
     	}
 
	if (tmp)                         // comp p no la tiene  
	     Candidates.remove(p-1);        // eliminar 
     }
  }



  // ahora creamos los editores.
  for (Candidates.first(); Candidates.current(); Candidates.next())
  {
	Editor = new TSignalProperty( Parent, Candidates.current() );
	  	
  	// ahora adicionamos todos los pares de componente::propiedad
	// a dicho editor
	for (objs.first(); objs.current(); objs.next() )
	{
  		Editor->props.append(objs.current()->obj, 
  		  objs.current()->cls->Prop(Candidates.current()) );
  	}
  	Editors.inSort(Editor);
  }  	
}



QString slotName(const char *s)
{
   char  tmp[256], *q;
   
   if (s) strcpy(tmp, s); else tmp[0] = 0;
   q = index(tmp, '(');
   if (q) *q = 0;
   return  tmp;
}

QString slotProto(const char *s)
{
   char  tmp[256], *q;
   
   strcpy(tmp, s);
   q = index(tmp, '(');
   if (q) return q;
   return "()";
}


void TSignalProperty::SetValue(const char *Value)
{
	for (uint i=0; i<props.count(); i++)  SetSignalValueAt(i, Value);
}

QString TSignalProperty::GetValue()
{
	QString Value = GetSignalValueAt(0);

	return Value;
}

class TO: public QObject {
public:
	QConnectionList *receivers(const char *s) { return QObject::receivers(s); }
};


bool internalGetSignal(QObject *o, QString Signal, QSignalProp &Value, QStrList Slots)
{ 
	QListT<QConnection> *list = (QListT<QConnection>*) ((TO*)o)->receivers(QString("2") + Signal);
	
	if (!list) return false;
	
	for (list->first(); list->current(); list->next())
	if (list->current()->object() == objectInspector->designer &&
	    Slots.find(slotName(list->current()->memberName())) != -1 )
	{
		Value.Receiver = list->current()->object();
		Value.Slot = list->current()->memberName();
		return true;
	}

	return false;
}

QString TSignalProperty::GetSignalValueAt(int Index) 
{ 
	QSignalProp	Value;
	QStrList 	Slots;
	GetValues(Slots);

	if (aProp(Index)) 
		Value = getSignalProp(aInst(Index), aProp(Index));
	else 
		internalGetSignal((QObject*) aInst(Index), Name + Proto, Value, Slots);
	
	return slotName(Value.Slot);
}

void	TSignalProperty::SetSignalValueAt(int Index, const char *Signal)
{
	QString lastValue = GetValue();
	QString newValue  = Signal;

	QStrList	Slots;	
	GetValues(Slots);

	if (newValue != "") 
	{
		if (Slots.find(newValue) == -1)
		if (lastValue == "")
			objectInspector->designer->CreateSlot(newValue+Proto);
		else
			objectInspector->designer->RenameSlot(lastValue+Proto, newValue+Proto);
	}


	// es el mismo valor ?
	if (lastValue == newValue)  return;

	if (aProp(Index)) 
	{
		setSignalProp(aInst(Index), aProp(Index), 
		    QSignalProp(objectInspector->designer, newValue+Proto));
		return;
	}
	
	// desconectar todo lo que este enganchado a esa sennal
	do {
		QSignalProp  Value;
		if (!internalGetSignal((QObject*) aInst(Index), Name + Proto, Value, Slots)) break;
		objectInspector->disconnect((QObject*) aInst(Index), QString("2") + Name + Proto,
		  (QObject*) Value.Receiver, QString("1") + Value.Slot);
	} while (1);

	if (newValue == "") return;
	
	// conectar ahora el slot
	objectInspector->connect((QObject*) aInst(Index), QString("2") + Name + Proto,
	  objectInspector->designer->Widget(), QString("1") + newValue + Proto);
	
}

TSignalProperty::TSignalProperty(TPropertyEditor *Parent, char* type) 
{  
   Index = 0;
   Index = Parent ? Parent->Index + 1 : 1;
   Name = slotName(type);
   Proto = slotProto(type);
}


const char*	TSignalProperty::name()
{
	return Name;
}



void TSignalProperty::GetValues(QStrList &Values)
{
	QMetaObject *m;

	if (!objectInspector->designer || !(m = objectInspector->designer->metaObject())) return;

	for (int i=0; i<m->numSlots(); i++)
 	if (m->slot(i) && (Proto == slotProto(m->slot(i)->name)) )
	 {
	        if (m->slot(i)->name && *m->slot(i)->name)
		 	Values.append(slotName(m->slot(i)->name));
	 }
	 
}

void TEnumProperty::SetStrValueAt(int Index, const char *Value)
{
	PropInfo	*prop = aProp(Index);
	EnumInfo 	*info = ( prop && (prop->type->kind == tkenum)) ? (EnumInfo*) prop->type : NULL;

	if (!info) return;
	
	for (EnumPair *pair = info->names; pair; pair = pair->next)
	 {
	 	if ( (Value == pair->name) || (Value && !strcmp(Value, pair->name)) ) 
	 	{
	 		setOrdProp(aInst(Index), prop, pair->value);
	 		return ;
	 	}
	 }
	
	int value;
	if (sscanf(Value, "%d", &value)==1) setOrdProp(aInst(Index), prop, value);

}

static char enumValue[64];

QString TEnumProperty::GetStrValueAt(int Index)
{
	PropInfo	*prop = aProp(Index);
	EnumInfo 	*info = ( prop && (prop->type->kind == tkenum)) ? (EnumInfo*) prop->type : NULL;

	if (!info) return "";
	
	int  Value = getOrdProp(aInst(Index), prop);
	
	for (EnumPair *pair = info->names; pair; pair = pair->next)
	 {
	 	if (Value == pair->value) return pair->name;
	 }
	
	return enumValue;
}

void TEnumProperty::GetValues(QStrList &Values)
{
	PropInfo	*prop = aProp(0);
	EnumInfo 	*info = ( prop && (prop->type->kind == tkenum)) ? (EnumInfo*) prop->type : NULL;

	if (!info) return;
	
	for (EnumPair *pair = info->names; pair; pair = pair->next)
	 {
	 	Values.append(pair->name);
	 }
}

