/****************************************************************************
    dsgnintf.h  30/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/
#ifndef _dsgnintf_
#define _dsgnintf_

#include <qlist.h>
#include <qobject.h>
#include <qmessagebox.h>
#include <qstrlist.h>
#include <stdio.h>

#include <qtrtti.h>
#include "inspector.h"


typedef struct TInstProp {
	void		*instance;
	PropInfo	*propinfo;
	TInstProp(void *i, PropInfo *p): instance(i), propinfo(p) {}
};

class InstPropList: public QListT<TInstProp> {
public:
	InstPropList(): QListT<TInstProp>() { setAutoDelete(true); }
	void append(void *i, PropInfo *p) { QListT<TInstProp>::append( new TInstProp(i, p) ); }
};

typedef enum {paValueList=1, paSubProperties=2, paDialog=4,
              paMultiSelect=8, paAutoUpdate=16, paSortList=32, 
              paReadOnly=64, paRevertable=128} TPropertyAttribute;

#define aProp(i) (props.at(i)->propinfo)
#define aInst(i) (props.at(i)->instance)

class TPropertyEditor;

class TPropertyEditorList: public QListT<TPropertyEditor> {
protected:
	int compareItems(void*, void*);
public:
	void clear();
};

class TPropertyEditor {
public:
	int			Index;
	TPropertyEditorList	SubProperties;
	InstPropList	 	props;

	virtual ~TPropertyEditor() { SubProperties.clear(); props.clear(); }
	virtual const char *name() { return (props.first() ? aProp(0)->name : ""); }
    	virtual QString GetValue() { return ""; }
    	virtual void GetValues(QStrList &Values) { }
	virtual void SetValue(const char *Value) { }
	virtual bool AllEquals() { return TRUE; }
	virtual int GetAttributes() { return paMultiSelect; }
	virtual int Edit() { return 0; }
	virtual void GetProperties(TPropertyEditorList &Editors) { }
protected:
//********************************** Ordinal Property Functions 
	virtual int GetOrdValueAt(int Index) 
	{ return (props.at(Index) ? getOrdProp(aInst(Index), aProp(Index)) : 0); }
	virtual void SetOrdValueAt(int Index, int Value) 
	{ setOrdProp(aInst(Index), aProp(Index), Value); }

	int	GetOrdValue();
	void	SetOrdValue(int Value);

//********************************** String "char*" Property Functions 
	virtual QString GetStrValueAt(int Index) 
	{ return (props.at(Index) ? getStrProp(aInst(Index), aProp(Index)) : QString("")); }
	virtual void SetStrValueAt(int Index, const char *Value) 
	{ setStrProp(aInst(Index), aProp(Index), Value); }

	QString GetStrValue();
	void	SetStrValue(const char *Value);

};	

class TOrdinalProperty : public TPropertyEditor
{
public:
    	QString GetValue() 
    	{ static char tmp[32];
    	  sprintf(tmp, "%d", GetOrdValue());
    	  return tmp; 
    	}
	void SetValue(const char *Value) 
    	{ int  v;
    	  sscanf(Value, "%d", &v);
    	  SetOrdValue(v);
    	}
	bool  AllEquals() 
	{ int tmp = GetOrdValueAt(0);
	  for (uint i=1; i<props.count(); i++) 
	   if (tmp!=GetOrdValueAt(i)) return FALSE;
	  return TRUE;
	}
};



class TStringProperty : public TPropertyEditor
{
public:
	virtual int GetAttributes() { return paAutoUpdate | paMultiSelect; }
    	QString GetValue() { return GetStrValue(); }
	virtual void SetValue(const char *Value) { SetStrValue(Value); }
	bool    AllEquals() 
	{ QString tmp = GetStrValueAt(0);
	  for (uint i=1; i<props.count(); i++) 
	   if (strcmp(tmp, GetStrValueAt(i))) return FALSE;
	  return TRUE;
	}
};

// las classes que no sean QObject (saca como subpropiedades )
class TClassProperty : public TPropertyEditor
{
public:
	virtual int GetAttributes() { return paMultiSelect | paSubProperties | paReadOnly; }
    	QString GetValue() { return aProp(0)->type->name; }
	void GetProperties(TPropertyEditorList &Editors);
};


class TSignalProperty : public TStringProperty
{
	QString		Name, Proto, Value;
public:
	TSignalProperty(TPropertyEditor *Top, char* type);
	virtual const char *name();
	virtual int GetAttributes() { return paValueList; }
// ============= signal managment
	virtual void SetValue(const char *Value);
    	QString GetValue();
    	virtual void GetValues(QStrList &Values);
//********************************** Signal Property Functions 
	virtual QString GetSignalValueAt(int Index);
	virtual void SetSignalValueAt(int Index, const char *Signal);

};

class TEnumProperty : public TStringProperty
{
public:
	virtual int GetAttributes() { return paValueList; }
    	virtual void GetValues(QStrList &Values) ;
    	
    	QString GetStrValueAt(int);
	virtual void SetStrValueAt(int, const char *);
};

class TSetProperty : public TOrdinalProperty
{
public:
	virtual int GetAttributes() { return paMultiSelect | paSubProperties | paReadOnly; }
	void GetProperties(TPropertyEditorList &Editors);
};

class TSetItemProperty : public TEnumProperty
{
public:
	TSetProperty	*set;	
	QString		Name;
	TSetItemProperty(TSetProperty *, const char *);
	const char *name() { return Name; }
    	QString GetStrValueAt(int);
	void SetStrValueAt(int, const char *);
    	virtual void GetValues(QStrList &Values) ;
};


template<class Type> class PropEditorClass {
public:
	char *name;
	PropEditorClass(char *nm): name(nm) {  }
	virtual Type *Create(TPropertyEditor *Top) 
	{  Type *tmp= new Type();
//	   tmp->inspector = i;
	   tmp->Index = 0;
//	   tmp->TopLevel = Top ? Top : tmp;
	   tmp->Index = Top ? Top->Index + 1 : 1;
	   return tmp;
	}
};

typedef PropEditorClass<TPropertyEditor> *PropertyEditorClass;


class PropertyEditorMapper;

class PropertyEditorMapper {
public:
	PropertyEditorMapper(const char *, const char *, void *);
public:
	PropertyEditorMapper *next;
	const char *Class;
	const char *Prop;
	void 	   *editor;
};

extern PropertyEditorMapper *mapperList;

#define RegisterPropertyEditor(O, P, E) \
PropertyEditorMapper O##P##E(#O, #P, (void*) &_##E)

#define RegisterClassEditor(O, E) \
PropertyEditorMapper O##E(#O, "", (void*) &_##E)


extern "C" {


void GetComponentProperties(TPropertyEditor *, InspectorDataList &, TPropertyEditorList &);
void GetComponentSignals(TPropertyEditor *, InspectorDataList &, TPropertyEditorList &);

PropertyEditorClass GetClassEditor(const char *Class);

}

#endif
