/****************************************************************************
    inspector.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "inspector.h"
#include "properties.h"

#include <qcombobox.h>
#include <qtabnote.h>

#include <qtrtti.h>

ObjectInspector *objectInspector = NULL;

ObjectInspector *ObjectInspector::create(QWidget *Parent)
{
	if (!objectInspector) new ObjectInspector(NULL);
	return objectInspector;
}

void ObjectInspector::free()
{
	if (objectInspector) delete objectInspector;
	objectInspector = NULL;
}

ObjectInspector::ObjectInspector(QWidget *Parent)
: QMainWindow(NULL)
{
	objectInspector = this;

	designer = NULL;

	components = new QComboBox(this);
	components->setGeometry(0, 0, width(), 24);

	pages = new QTabNote(this);
	propPage = new TPropsInspector(pages);
	pages->addTab(propPage, "Props");
	signalPage = new TPropsInspector(pages);
	pages->addTab(signalPage, "Signals");


	setGeometry(0, 130, 200, 400);
	setCaption("Object Inspector");
}

bool ObjectInspector::queryClose()
{
	hide();
	return FALSE;
}

void ObjectInspector::resizeEvent(QResizeEvent *e)
{ 
	components->setGeometry(0, 0, width(), components->height());
	pages->setGeometry(0, 24, width(), height() - components->height());
	QMainWindow::resizeEvent(e);
}


void ObjectInspector::EditProperties(FormDesigner *Form, void *user, InspectorDataList &o)
{
	objs.clear();
	propPage->EditProperties(objs);
	signalPage->EditSignals(objs);

	designer = Form;
	userData = user;

	if (o.count()>0)
	for (o.first(); o.current(); o.next())
	  objs.append(o.current()->obj, o.current()->cls);

	propPage->EditProperties(objs);
	signalPage->EditSignals(objs);
}

/*
void ObjectInspector::GetChildList(QObject *top)
{

	if (!top) return;
	
	if (top->children())
	{
		QListT<QObject> *children = (QListT<QObject> *) top->children();
		for (children->first(); children->current(); children->next())
		if (children->current()->name() &&
		    *children->current()->name() &&
		    ClassInfo::Ancestor(children->current()) ) 
		{
			components->insertItem( 
				QString(children->current()->name()) + ": " + 
				children->current()->className());
		        GetChildList( children->current() );
		}
	}
} 


void ObjectInspector::getComponentList(QObject *top, const QString Name)
{
   components->clear();
   if (top)
   {
	QString tmp;
	if (Name.length()>0) tmp = Name;
	else tmp = QString(top->name())+ ": " + top->className();
	components->insertItem( tmp );
        GetChildList(top);
   }
}

*/
