/****************************************************************************
    properties.cpp  24/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "inspector.h"
#include "properties.h"
#include "dsgnintf.h"
#include "dsgnedit.h"
#include <qtrtti.h>


#include <qpainter.h>

#define lineHeight  20

#define H(i) ((i)*lineHeight)	 
#define ERect(i) QRect(104, H(i-Origin), width()-101, lineHeight) 
#define Editor(i) ((i>-1) && (i<(int)Editors.count()) ? Editors.at(i) : NULL)

#include "../mainwnd.h"



int TPropertyEditorList::compareItems(void *item1, void *item2)
{
	return strcmp( ((TPropertyEditor*)item1)->name(), ((TPropertyEditor*)item2)->name() ); 
}

void TPropertyEditorList::clear()
{
	while (first()) delete take(0);
}


TPropsInspector::TPropsInspector(QWidget *p): QWidget(p)
{
	propEdit = NULL;
	setGeometry(0, 130, 200, 400);
}


void TPropsInspector::OnClickSlot()
{
	mainWindow->SetModify(objectInspector->designer);
	if (propEdit->Modified()) propEdit->Sync();
}

void TPropsInspector::OnEditSlot()
{
	mainWindow->SetModify(objectInspector->designer);
	propEdit->Edit();
}

void TPropsInspector::OnTextSlot(const QString &)
{
	if (Editor(Cursor)->GetAttributes() & paAutoUpdate) 
	{
		propEdit->Sync();
		if (!propEdit->Modified()) mainWindow->SetModify(objectInspector->designer);
	}	
	propEdit->modified = 1;
}

void TPropsInspector::OnListSlot(const QString&)
{
	mainWindow->SetModify(objectInspector->designer);
	propEdit->Sync();
	
	QStrList  Values;
	Editor(Cursor)->GetValues(Values);

	((QComboBox*) propEdit->Widget())->clear();

	for (Values.first(); Values.current(); Values.next())
	((QComboBox*) propEdit->Widget())->insertItem(Values.current());

	propEdit->Update();
}



void TPropsInspector::paintEvent(QPaintEvent *e)
{
	QPainter  painter(this);
	QBrush    ltGrayBrush(lightGray);
	QBrush    WhiteBrush(white);
	int	  i = 0;
	char	  tmp[128];
	
	for (Editors.first(); Editors.current(); Editors.next(), i++)
	{
		TPropertyEditor *p = Editors.current();

		painter.fillRect(0, H(i), width(), lineHeight, ltGrayBrush);
		
		painter.setPen(white);
		painter.drawLine(0, H(i+1)-1, 0, H(i));
		painter.drawLine(0, H(i), 99, H(i));
		
		painter.setPen(darkGray);
		painter.drawLine(99, H(i), 99, H(i+1)-1);
		painter.drawLine(0, H(i+1)-1, width(), H(i+1)-1);

		painter.setPen(black);
		memset(tmp, 32, (p->Index-1)*3);
		sprintf(tmp+(p->Index-1)*2, "%s%s", 
		 p->GetAttributes() & paSubProperties ? 
		 (p->SubProperties.count() ? "-" : "+") : 
		 "  "
		 , p->name());	 
		painter.drawText(4, H(i), 100 - 4, lineHeight, AlignLeft | AlignVCenter, tmp);

		painter.drawText(ERect(i), AlignLeft | AlignVCenter, 
		 p->GetValue() ); 
	}
}

void TPropsInspector::CursorTo(int Pos)
{	int o = Origin;
 	int c = Cursor;
 	
 	if ((Pos<0) || (Pos+1> (int) Editors.count()) ) return;
 	
	Cursor = Pos;
//	Origin = MAX(MIN(Origin, Cursor), Cursor - getHeight()/20);
	
	{
	QRect  R = ERect(Cursor);
	
	if (c!=Cursor) 	EditorOff();


	if (Editor(Cursor)->GetAttributes() & paValueList)
	// Create a value list editor
	{
		if (c!=Cursor) 	
		{
			QStrList  Values;
			Editor(Cursor)->GetValues(Values);
			propEdit = new TListEdit(this, Editor(Cursor));
			for (Values.first(); Values.current(); Values.next())
			((QComboBox*) propEdit->Widget())->insertItem(Values.current());
			propEdit->Update();
		}
		propEdit->Widget()->setGeometry(R.x()-4, R.y()-1, R.width()+2, R.height()+2);
	
	} else
	if (Editor(Cursor)->GetAttributes() & paDialog)
	// Create a "..." button
	{
		if (c!=Cursor) 	propEdit = new TDialogEdit(this, Editor(Cursor));
		propEdit->Widget()->setGeometry(width()-30, R.y(), 30, R.height()-1);

	} else
	// Create a line editor
	{	
		if (c!=Cursor) 	propEdit = new TLineEdit(this, Editor(Cursor));
		propEdit->Widget()->setGeometry(R.x()-4, R.y()-1, R.width()+2, R.height()+2);

	}
	if (propEdit) 
	{
		propEdit->Widget()->show();
		if (Editor(Cursor)->GetAttributes() & paReadOnly)	
			propEdit->Widget()->setEnabled(FALSE);
	}
	}
	if (o!=Origin) repaint(TRUE);
//	if (propEdit) propEdit->Widget()->setFocus();
}

void Collapse(TPropertyEditorList &Editors, TPropertyEditor &Edit)
{
	if (!Edit.SubProperties.count()) return;
	for (Edit.SubProperties.first();Edit.SubProperties.current();Edit.SubProperties.next())
	{
		Collapse(Editors, *Edit.SubProperties.current());
	        Editors.remove(Edit.SubProperties.current());
	}
	Edit.SubProperties.clear();
}

void TPropsInspector::EditProperties(InspectorDataList &objs)
{ 

  int i;
  EditorOff();
  i=0;
  while (i < (int) Editors.count())
  {	
  	Collapse(Editors, *Editor(i));
 	i++; 
  }
  Editors.clear(); 
  Origin = 0;
  Cursor = -1;

  GetComponentProperties(NULL, objs, Editors);
  repaint(TRUE);
  CursorTo(0);
}

void TPropsInspector::EditSignals(InspectorDataList &objs)
{ 

  int i;
  EditorOff();
  i=0;
  while (i < (int) Editors.count())
  {	
  	Collapse(Editors, *Editor(i));
 	i++; 
  }
  Editors.clear(); 
  Origin = 0;
  Cursor = -1;

  GetComponentSignals(NULL, objs, Editors);
  repaint(TRUE);
  CursorTo(0);
}


void TPropsInspector::EditorOff()
{
	if (!propEdit) return;
	if (propEdit->Modified()) 
	{
		mainWindow->SetModify(objectInspector->designer);
		propEdit->Sync();
	}
	delete propEdit;
	propEdit = NULL;
}

void TPropsInspector::resizeEvent(QResizeEvent *e)
{ 
	QWidget::resizeEvent(e);
	CursorTo(Cursor);
}

void TPropsInspector::mousePressEvent(QMouseEvent *e)
{ 
	QWidget::mousePressEvent(e);
	CursorTo(Origin + e->y() / lineHeight);
}

void TPropsInspector::mouseDoubleClickEvent(QMouseEvent *e) 
{ 
	TPropertyEditor *ed = Editor(Cursor); 
	QWidget::mouseDoubleClickEvent(e);
	if (ed)
	if (ed->GetAttributes() & paSubProperties) 
	{
		if (ed->SubProperties.count())
		{
			Collapse(Editors, *ed);
		        repaint(TRUE);
		} else
		{
			int i=1;
			ed->GetProperties(ed->SubProperties);
			for (ed->SubProperties.first();ed->SubProperties.current();ed->SubProperties.next(), i++)
		        Editors.insert(Cursor+i, ed->SubProperties.current());
		        repaint(TRUE);
		}
	} else Editor(Cursor)->Edit();
}



