/****************************************************************************
    mainwnd.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include <qfiledialog.h>
#include "mainwnd.h"
#include <qframe.h>
#include <qtabbar.h>
#include "formedit/designer.h"
#include "srcedit/srcedit.h"
#include "inspector/inspector.h"
#include "files.h"
#include <qmessagebox.h>
#include <qregexp.h>
#include <stdarg.h>
#include "dlgs/viewdlg.h"
#include "unistd.h"
#include "strutils.h"
#include "srcmaker.h"
#include <dlfcn.h>

#include "images/resources.cpp"

#include "source.h"

extern "C" int ShellExecute(QString Command, void (*getStd1)(QString), void (*getStd2)(QString));
extern "C" bool CompileProject(QString);

extern QString exePath;
extern QMenuBar *menu;


class QSource: public QTab {};



MainWindow  		*mainWindow = NULL;
QListT<Source>		sources;
QListT<FormDesigner>	forms;
QStrList		projectFiles;
Source			*projectPath = NULL;
Source			*projectWindow = NULL;

bool FindRef(FormDesigner **form, Source **cpp, Source **inc)
{
	if (form && *form) // buscar las fuentes de la forma
	for ( sources.first(); sources.current(); sources.next() )
	if (sources.current()->form == *form)
	{
		if (cpp) *cpp = NULL; 
		// chequeo por si acaso
		if (sources.current()->ext == ".cpp")
		{
			if (cpp) *cpp = sources.current();

			QListT<Source> &l = (*cpp)->depending; 

			if (inc) *inc = NULL;
			for (l.first(); l.current(); l.next())
			if (l.current()->ext == ".h")
			{
				if (inc) *inc = l.current();
				break;
			}
			
			break;
		}
		return (!cpp || *cpp) && (!inc || *inc);
	}
	
	if (cpp && *cpp) // buscar la forma y el include 
	{
		if (form) *form = (*cpp)->form;

		QListT<Source> &l = (*cpp)->depending; 

		if (inc) *inc = NULL;
		for (l.first(); l.current(); l.next())
		if (l.current()->ext == ".h")
		{
			if (inc) *inc = l.current();
			break;
		}
		return (!form || *form) && (!inc || *inc);
	}
	
	if (inc && *inc) // buscar la forma y el include 
	{
		if (cpp) *cpp = (*inc)->main;
		
		if (form && (*inc)->main) *form = (*inc)->main->form;
		
		return (!cpp || *cpp) && (!form || *form);
	}	
	
	return 0;
}



Source::Source(Source *Main, QString Text)
: modified(0), project(0), filename(""), edit(0), main(Main), form(0)
{
	depending.setAutoDelete(false);
	text = Text;
	sources.append(this);
	if (main) main->depending.append(this);
}

Source::~Source()
{
	while ( depending.last() ) 
	{
		Source *s = depending.last();
		depending.remove( s );

		// delete it
		sources.remove( s );
	}
	if (edit) 
	{
		sourceEditor->ClosePage(edit);
		edit = NULL;
	}
	if (form) forms.remove(form);
}


bool	Source::Modified()
{
	if (
	    modified || 
	    ( sourceEditor->modified && edit && (edit == sourceEditor->source) ) 
	) return true;

	for (depending.first(); depending.current(); depending.next())
	if (depending.current()->Modified()) return true;

	return false;
}

bool    Source::Find(int &pos, int &len, const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	char	tmp[2000];
	vsprintf(tmp, fmt, ap);
	va_end(ap);
	
	sourceEditor->UpdateContents(edit);

	QRegExp  find(tmp);
	
	pos = find.match(text, pos, &len);

	return  (pos != -1);
}

void	Source::Replace(int pos, int len, const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	char	tmp[2000];
	vsprintf(tmp, fmt, ap);
	va_end(ap);
	
	sourceEditor->UpdateContents(edit);

	if (len>0) text.remove(pos, len);
	text.insert(pos, tmp);
	
	modified = 1;
	
	sourceEditor->UpdateSource(edit);
}

void	Source::ReplaceText(const char *s1, const char *s2)
{
	int len, pos = 0;

	if (Find(pos, len, s1)) Replace(pos, len, s2);
}





PushButton *doTool(QWidget *P, char **img, int X, int Y, char *, char *, char *hint)
{
  	PushButton *btn = new PushButton(P, hint, hint);
	
	if (img && *img) btn->setPixmap(QPixmap((const char **) img)); 
	btn->setGeometry(X, Y, 29, 29); 
	btn->setFixedSize(29, 29);
	
	return btn;
}

#include <qlabel.h>

class TPaletteItem: public QWidget {
public:
	PushButton	*Pointer;
	TPaletteItem(QWidget *p, char *nm): QWidget(p, nm)
	 /*, QFrame::Panel | QFrame::Raised)*/, Pointer(NULL) {}
};	

#define Children(W) (W->children() ? ((QListT<QObject> *) (W->children()))->count() : 0)

#include "component.xpm"

#include <qtabnote.h>

void  createTools(QTabNote *tabs)
{
	if (!componentList) return;

	for (componentList->first(); componentList->current(); componentList->next() )
	if (componentList->current()->Create)
	{
		PushButton *btn;
		TPaletteItem *pal = (TPaletteItem*) tabs->Page(componentList->current()->palette);

		if (!pal) 
		{
			pal = new TPaletteItem(tabs, componentList->current()->palette);
			tabs->addTab(pal, componentList->current()->palette);
			pal->Pointer = doTool(pal, pointer, 5, 5, 
				NULL, NULL, "Pointer");
			pal->Pointer->connect(pal->Pointer, SIGNAL(OnClick(PushButton*)), 
				mainWindow, SLOT(ToolClicked(PushButton*)));
		}
		char **img = componentList->current()->img ?
			     componentList->current()->img :
			     ComponentXPM;

		btn = doTool(pal, img,
                           5+10+29*Children(pal), 5, 
			   NULL, NULL, componentList->current()->classname);
		btn->connect(btn, SIGNAL(OnClick(PushButton*)), 
			mainWindow, SLOT(ToolClicked(PushButton*)));
			
	}
}

#include <qpopupmenu.h>
#include <qmenubar.h>


static QTabNote  *pages = NULL;
QMenuBar *menu = NULL;



MainWindow::MainWindow()
: QMainWindow(NULL)
{
	mainWindow = this;
	QWidget	*d = QApplication::desktop();

	sources.setAutoDelete(true);
	forms.setAutoDelete(true);

	menu = new QMenuBar(this);
	
  QPopupMenu* p;
  int id;

  p = new QPopupMenu;
  id = p->insertItem("&New project", this, SLOT(NewProject()));
  id = p->insertItem("&Open project...", this, SLOT(OpenProject()));
  id = p->insertItem("&Open file...", this, SLOT(OpenFile()));
  id = p->insertItem("New form", this, SLOT(NewForm()));
  id = p->insertItem("&Save file...", this, SLOT(SaveFile()));
  p->insertSeparator();
  id = p->insertItem("Save All", this, SLOT(SaveAll()));


  id = p->insertItem(("Close All"), this, SLOT(CloseAll()));
  p->insertSeparator();
  id = p->insertItem(("&Quit"), this, SLOT(Quit()));

  menu->insertItem(("&File"), p);

  p = new QPopupMenu;
  id = p->insertItem(("&Compile..."), this, SLOT(Compile()));
  id = p->insertItem(("Make &dependences"));
  id = p->insertItem(("Make &clean"));
  id = p->insertItem(("Build &all"));
  p->insertSeparator();
  id = p->insertItem(("&Run"), this, SLOT(RunProject()));

  menu->insertItem(("&Compile"), p);



     	connect(doTool(this, openPrj, 3, 30, NULL, NULL, "Open Project"),
        SIGNAL(clicked()), SLOT(OpenProject()));
     	connect(doTool(this, savePrj, 32, 30, NULL, NULL, "Save All"),
     	SIGNAL(clicked()), SLOT(SaveAll()));
     	doTool(this, addPrj, 61, 30, NULL, NULL, "Add to Project");
     	connect(doTool(this, viewSource, 90, 30, NULL, NULL, "View Source"),
     	SIGNAL(clicked()), SLOT(ViewSource()));
     	connect(doTool(this, viewForm, 119, 30, NULL, NULL, "View Window"),
     	SIGNAL(clicked()), SLOT(ViewWindow()));
     	connect(doTool(this, openFile, 3, 60, NULL, NULL, "Open file"),
        SIGNAL(clicked()), SLOT(OpenFile()));
     	connect(doTool(this, saveFile, 32, 60, NULL, NULL, "Save File"),
        SIGNAL(clicked()), SLOT(SaveFile()));
      	doTool(this, removePrj, 61, 60, NULL, NULL, "Remove from Project");
     	connect(doTool(this, newForm, 90, 60, NULL, NULL, "New Window"),
        SIGNAL(clicked()), SLOT(NewForm()));
     	connect(doTool(this, runPrj, 119, 60, NULL, NULL, "Run Project"),
        SIGNAL(clicked()), SLOT(RunProject()));
     	

	pages = new QTabNote(this);
	pages->connect(pages, SIGNAL(selected(int)), this, SLOT(PageChanged(int)));

	setGeometry(0, 0, d->width(), menu->height() + 2 * 29 + 10);
	setFixedSize(width(), height());

	lastTool = NULL;
}

extern "C" void loadComponents();

void loadComponents()
{
	createTools(pages);
}

void  MainWindow::resizeEvent(QResizeEvent *e)
{
	QMainWindow::resizeEvent(e);
	pages->setGeometry(200, menu->height(), width()-200, height()-menu->height());
}


void  MainWindow::ToolClicked(PushButton *btn)
{
	if (!btn)
	{
		TPaletteItem *pal = (TPaletteItem*) pages->Page(pages->page());
		
		if (pal) btn = pal->Pointer;
	}

	if (btn!=lastTool)
	{ 
		if (lastTool) 
		{
			lastTool->setOn(FALSE);
			lastTool->setToggle(FALSE);
		}
		lastTool = btn;
	}
	
	if (lastTool) 
	{
		lastTool->setToggle(TRUE);
		lastTool->setOn(TRUE);
	}
}

void MainWindow::PageChanged(int i)
{	
	ToolClicked(NULL);
}




void Source::Save(QString Path, QString Name)
{
	sourceEditor->UpdateContents(edit);
	
	if (filename != (Path+Name+ext))
	{
		if (ext == ".cpp")
		{
			// si es un cpp que incluye un .h de igual nombre
			// actualizar el valor del #include "nombre.h"
			ReplaceText(
				Format("\x22%s.h\x22", name.data()),
			        Format("\x22%s.h\x22", Name.data())
        		);
		} else
		if (ext == ".h")
		{
			// actualizar el valor del #define _name_
			ReplaceText(
				Format("#ifndef _%s_[^\\n]*\\n#define _%s", name.data(), name.data()), 
				Format("#ifndef _%s_\n#define _%s", Name.data(), Name.data())
			);
	
			if (projectPath)
			projectPath->ReplaceText(
				Format("\x22%s.h\x22", name.data()),
				Format("\x22%s.h\x22", Name.data())
			);
			        
		}
	        name = Name;
	        filename = Path+Name+ext;

	        // poner el label del editor bien puesto
		// xxx..x..x..x...x.xxxx
		if (edit) 
		{
			edit->label = name+ext;
			sourceEditor->Realign();
		}
	}

	saveASCIIFile(filename, text);
	modified = false;
	
	if (!main && project && (projectFiles.find(filename) == -1))
	projectFiles.append(filename);
		
	
	sourceEditor->UpdateSource(edit);
}

bool saveSource(Source *s)
{
	if (s->main) return saveSource(s->main);

	if (!s->Modified()) return true;
	
	QString FileName = s->filename == "" ?  
			   getSaveFileName(s->name+s->ext, "*.cpp") : 
			   s->filename;
			   
	if (FileName.isNull()) return false;

	QString Path, Name, Ext;

	fileSplit(FileName, Path, Name, Ext);


	for (s->depending.last(); s->depending.current(); s->depending.prev())
	if (s->depending.current()->modified)
	{
		s->depending.current()->Save(Path , Name);
	}
	
	s->Save(Path, Name);
	
	if (s->form) s->form->SaveRes(Path, Name);
	 	
	if (s == projectPath) mainWindow->SaveProject();

	return true;
}


//========================================================================

void MainWindow::CloseAll()
{
	Source	*s;
	
	for (sources.last(); sources.current(); sources.prev())
	if (!sources.current()->main && sources.current()->Modified())
	{
		s = sources.current();
		
		int action = QMessageBox::warning(this, "Fuente modificada", 
   	       		Format("Desea guardar los cambios realizados en %s?", (s->name+s->ext).data()),
			  3, 4, 2); // yes no cancel
				
		// cancel ?
		if (action==2) return; 
		
		// save ?
		if ((action==3) && !saveSource(s)) return;
	}
	
	while ( sources.first() ) sources.remove( sources.first() );
	projectPath = NULL;
	projectWindow = NULL;
        projectFiles.clear();
}

void MainWindow::SaveProject()
{
	if (projectPath)
	{
		QString  Path, SRCS, OBJS, MOCS, RCS;
		
		if (projectWindow) 
		{
			projectFiles.remove(projectWindow->filename);
			projectFiles.insert(0, projectWindow->filename);
		}
		projectFiles.remove(projectPath->filename);
		projectFiles.insert(0, projectPath->filename);
		
		fileSplit(projectPath->filename, Path, projectPath->name, projectPath->ext);

		FILE  *f=fopen(Path+projectPath->name+".prj", "w");
		
		if (!f) return;
		
		for (projectFiles.first(); projectFiles.current(); projectFiles.next())
		{
			QString aPath, Name, Ext;
			fileSplitRel(projectFiles.current(), projectPath->filename, aPath, Name, Ext);
			
			fprintf(f, "%s\n", (aPath+Name+Ext).data() );
			
			SRCS = SRCS + aPath + Name + Ext + " ";
			OBJS = OBJS + aPath + Name + ".o ";
			if (fileExist(Path + Name + ".vqt"))
			{
				MOCS = MOCS + aPath + Name + ".moc ";
				RCS  = RCS  + aPath + Name + ".rc ";
			}			
		}
		
		fclose(f);
		

		f=fopen(Path+"Makefile", "w");
		
		if (!f) return;
	
		fprintf(f, "PROJECT = %s\n", projectPath->name.data() );
		fprintf(f, "SRCS = %s\n", SRCS.data());
		fprintf(f, "OBJS = %s\n", OBJS.data());
		fprintf(f, "MOCS = %s\n", MOCS.data());
		fprintf(f, "RCS = %s\n", RCS.data());
		
		fclose(f);		

		QString Data = loadASCIIFile(exePath+"../template/Makefile");
		saveASCIIFile(Path+"Makefile", Data, false);
	}
}


void MainWindow::SaveAll()
{
	SaveAll(false);
}


void MainWindow::SaveFile()
{
	QSource *qsource = sourceEditor->source;
	if (!qsource) return;

	for (sources.first(); sources.current(); sources.next())
	if (sources.current()->edit == qsource) break;
	
	Source *source = sources.current();
	if (!source) return;
	
	saveSource(source->main ? source->main : source);
}


void MainWindow::OpenProject()
{	
	QString filename = getOpenFileName("*.prj", "*.prj");

	if (filename.length()==0) return;
	CloseAll();
	if (sources.first()) return;

	FILE *f = fopen(filename, "r");

	if (f)
	{
		QString  Path, Name, Ext;
		char   tmp[256];
		fileSplit(filename, Path, Name, Ext);

		while (fscanf(f, "%s\n", tmp) == 1) 
		{
			if (
				fileExist(Path + tmp) && 
			    	(projectFiles.find(Path+tmp) == -1) 
			)
			    projectFiles.append(Path + tmp);
		}
		
		if (projectFiles.count()>0) 
		{
			projectPath = OpenFile(projectFiles.at(0), false);	
			if (projectFiles.count()>1) 
			{
				projectWindow = OpenFile(projectFiles.at(1), true);	
			}
		}
		fclose(f);
	}
}


static  int  fcount = 1;


void MainWindow::NewForm()
{	
	FormDesigner	*form = new FormDesigner(NULL);
	QString 	cpp, inc;

	form->setName(Format("QMainWindow%d", fcount));
	form->setCaption(Format("QMainWindow%d", fcount));
	QString Name = Format("qmainwindow%d", fcount);

	srcCreateWindow(form, Name, cpp, inc);
	fcount++;
	
	Source	*CPP = new Source(NULL, cpp);
	CPP->project = 1;
	CPP->modified = 1;
	CPP->name = Name;
	CPP->ext = ".cpp";
	CPP->form = form;
	if (!projectWindow) projectWindow = CPP;

	Source	*INC = new Source(CPP, inc);
	INC->modified = 1;
	INC->name = Name;
	INC->ext = ".h";
	
	forms.append(form);

	int len, pos;
	if (projectPath && projectWindow == CPP)
	{
		pos =0;
		while (projectPath->Find(pos, len, "FILE"))
		{
			projectPath->Replace(pos, len, "%s", CPP->name.data() ); 	
		}
		pos = 0;
		while (projectPath->Find(pos, len, "NAME"))
		{
			projectPath->Replace(pos, len, "%s", form->name() ); 	
		}
	}

	CPP->edit = 
  	  sourceEditor->NewPage(CPP->name + CPP->ext, CPP->text, CPP->modified);

	form->show();	
}

void MainWindow::closeEvent(QCloseEvent *e)
{
	CloseAll();
	if (sources.first()) 
	{
		e->ignore();
		return;
	}

	delete sourceEditor;
	objectInspector->free();

	e->accept();
	qApp->quit();
}

void MainWindow::KeyEvent(QWidget *Sender, QKeyEvent *e, void *data)
{
//	printf("%d %d %d\n", e->key(), e->ascii(), e->state());
	
	// Ctrl + S
	if ((e->state() & 16) && (e->key()==83)) SaveFile();

	// F12
	if (e->key() == 4155)
	if ( Sender == sourceEditor )
	{

	QSource *qsource = sourceEditor->source;
	if (!qsource) return;

	for (sources.first(); sources.current(); sources.next())
	if (sources.current()->edit == qsource) break;
	
	Source *cpp = sources.current();
	if (!cpp) return;
	
		if ( !cpp->form ) cpp = cpp->main;
		if ( cpp && cpp->form ) 
		{
			cpp->form->show();
			cpp->form->raise();
			cpp->form->setFocus();
		}
		
	} else
	if ( Sender == objectInspector )
	{
	
	} else  // Sender is FormDesigner.
	{

	
	}

}

#include "dlgs/newdialog.h"

void MainWindow::NewProject()
{
//        (new NewDialog(this))->exec();


	CloseAll();
	if (sources.first()) return;

	Source	*source = Source::Load(NULL, exePath+"../template/project.cpp");
	source->project = 1;
	source->filename = "";
	source->modified = true;
	projectPath = source;

	NewForm();

	projectFiles.clear();	
}

void MainWindow::CreateSlot(FormDesigner *Form, QString Name)
{
	Source  *cpp=NULL, *inc=NULL;
	
	if (FindRef(&Form, &cpp, &inc))
	{
		int len, pos = 0;

		if (inc->Find(pos, len, "public slots:[^\\n]*\\n"))
		{
			inc->Replace(pos+len, 0, "    void %s;\n", Name.data());
		}
		
		sourceEditor->UpdateContents(cpp->edit);
		cpp->text.append(QString("void ")+Form->Instance()->name()+"::"+Name+"\n{\n\n}\n");
		sourceEditor->UpdateSource(cpp->edit);
	}
}

void MainWindow::RenameSlot(FormDesigner *Form, QString Name, QString newName)
{
	Source  *cpp, *inc;
	
	if (FindRef(&Form, &cpp, &inc))
	{
		int len, pos = 0;

		if (inc->Find(pos, len, "void[\\s\\t]+%s", Name.data() ))
		{
			inc->Replace(pos, len, "void %s", newName.data());
		}
		
		pos = 0;
		if (cpp->Find(pos, len, "void[\\s\\t]+%s::%s", Form->Instance()->name(), Name.data()))
		{
			cpp->Replace(pos, len, "void %s::%s", Form->Instance()->name(), newName.data());
		}
	}
}

void MainWindow::designerAddComp(FormDesigner *Form, Designer *Parent, Designer *Child)
{
	ToolClicked(NULL);

	Source  *cpp, *inc;

	ComponentClass *Class = (*componentList)[Child->Instance()->className()];
	
	if (Class && FindRef(&Form, &cpp, &inc))
	{
		sourceEditor->UpdateContents(cpp->edit);
		sourceEditor->UpdateContents(inc->edit);
		int r = srcAddObject(Form, Child->Instance(), cpp->text, inc->text, Class->inc);
		if (r & 1) { inc->modified |= true; sourceEditor->UpdateSource(inc->edit); }
		if (r & 3) { cpp->modified |= true; sourceEditor->UpdateSource(cpp->edit); }
	}
}

void MainWindow::RenameComponent(FormDesigner *Form, Designer *Child, QString Name)
{
	Source  *cpp, *inc;
	if (!Form || !Child || Name.length()==0) return;
	
	if (FindRef(&Form, &cpp, &inc))
	if (Child->Instance() == Form->Instance())
	// cambiendo el nombre de la ventana
	{
		sourceEditor->UpdateContents(cpp->edit);
		sourceEditor->UpdateContents(inc->edit);
		int r = srcRenameWindow(Form, Name, cpp->text, inc->text);
		if (r & 1) { inc->modified |= true; sourceEditor->UpdateSource(inc->edit); }
		if (r & 3) { cpp->modified |= true; sourceEditor->UpdateSource(cpp->edit); }

		if (projectPath)
		{
			sourceEditor->UpdateContents(projectPath->edit);
			ReplaceAllStr(projectPath->text, Form->name(), Name);
			projectPath->modified |= true; 
			sourceEditor->UpdateSource(projectPath->edit); 			
		}
	} else
	{
		sourceEditor->UpdateContents(cpp->edit);
		sourceEditor->UpdateContents(inc->edit);
		int r = srcRenameObject(Form, Child->Instance(), Name, cpp->text, inc->text);
		if (r & 1) { inc->modified |= true; sourceEditor->UpdateSource(inc->edit); }
		if (r & 3) { cpp->modified |= true; sourceEditor->UpdateSource(cpp->edit); }
	}
}

void MainWindow::RemoveComponent(FormDesigner *Form, Designer *Child)
{
	Source  *cpp, *inc;
	
	if (FindRef(&Form, &cpp, &inc))
	{
		sourceEditor->UpdateContents(cpp->edit);
		sourceEditor->UpdateContents(inc->edit);
		int r = srcDeleteObject(Form, Child->Instance(), cpp->text, inc->text);
		if (r & 1) { inc->modified |= true; sourceEditor->UpdateSource(inc->edit); }
		if (r & 3) { cpp->modified |= true; sourceEditor->UpdateSource(cpp->edit); }
	}
}

void MainWindow::SetModify(FormDesigner *Form)
{
	Source  *cpp, *inc;
	
	if (Form && FindRef(&Form, &cpp, &inc))
	{
		cpp->modified = true;
		sourceEditor->modified = cpp->modified;
//		sourceEditor->status->changeItem(cpp->modified ? "Modified": "", 2);
	}
}

void	MainWindow::ViewWindow()
{

}



void	MainWindow::Quit()
{
	CloseAll();
	if (sources.first()) return;
	
	close();
}

Source  *Source::Load(Source *Main, QString FileName)
{
	for (sources.first(); sources.current(); sources.next())
	if (sources.current()->filename == FileName) 
	{
		if ( sources.current()->ext == ".h" )
			sources.current()->main = Main;
		return sources.current();
	}	
	
	Source *source = new Source(Main, loadASCIIFile(FileName));
	QString tmp;
	source->filename = FileName;
	fileSplit(FileName, tmp, source->name, source->ext);
	return source;
}


void MainWindow::OpenFile()
{	
	OpenFile(getOpenFileName("*.cpp", "*.cpp"), true);
}

Source *MainWindow::OpenFile(QString filename, bool Show)
{	
	if (filename.length() == 0) return NULL;
	
	
	QString	Path, Name, Ext;
	Source  *source;
		
	fileSplit(filename, Path, Name, Ext);
		
	source = Source::Load(NULL, Path+Name+Ext);
	if (!source) return NULL;
		
	if (Show) source->edit = 
		sourceEditor->NewPage(source->name+source->ext, source->text, source->modified);
		
	if ( ( Ext == ".cpp" ) && fileExist(Path+Name+".h") )
	{
		if (!Source::Load(source, Path+Name+".h")) return source;
			
		if (fileExist(Path+Name+".vqt"))
		{
			FormDesigner *form = new FormDesigner(NULL);
			if (form->LoadRes(Path, Name))
			{
				source->form = form;
				forms.append(form);
				if (Show) form->show();	
			} else delete form;
		}
	}
	return source;	
}

void MainWindow::CloseSource()
{
	QSource *qsource = sourceEditor->source;
	if (!qsource) return;

	for (sources.first(); sources.current(); sources.next())
	if (sources.current()->edit == qsource) break;
	
	Source *source = sources.current();
	if (!source) return;
	
	// si es dependiente cerrar pagina del editor, 
	// pero no borrar la fuente.
	if (source->main || (projectPath == source) )
	{
		sourceEditor->ClosePage(source->edit);
		source->edit = NULL;
		return;
	}
	
	if ( source->Modified() )
	{
		int action = QMessageBox::warning(this, "Fuente modificada", 
		     Format("Desea guardar los cambios realizados en %s?", (source->name+source->ext).data()),
			       3, 4, 2); // yes no cancel
				
		if (action==2) return; // cancel
		
		if ((action==3) && !saveSource(source)) return;
	}	

	sources.remove(source);
}

void	MainWindow::ViewSource()
{
	QStrList l;

	for (sources.first(); sources.current(); sources.next())
	l.append(sources.current()->name+sources.current()->ext);
	
	QString s = ViewDlg::ShowModal(this, l);

	if (s.length()>0)
	for (sources.first(); sources.current(); sources.next())
	if ( s == sources.current()->name+sources.current()->ext )
	if (sources.current()->edit) sourceEditor->pages->setCurrentTab(sources.current()->edit->id);
	else
	{
		sources.current()->edit = 
 	 		sourceEditor->NewPage(s, sources.current()->text, sources.current()->modified);
	}
}

void	MainWindow::Compile()
{
	if (!projectPath || !SaveAll(false)) return;
	SaveProject();

	QString  Path, Name, Ext;

	fileSplit(projectPath->filename, Path, Name, Ext);

//	ShellExecute(exePath + "make.sh %s", Path.data() );
}


void	MainWindow::RunProject()
{
	if (!projectPath || !SaveAll(false)) return;

	SaveProject();

	QString  Path, Name, Ext;

	fileSplit(projectPath->filename, Path, Name, Ext);

	if ( CompileProject(Path) )
	{
		if ( ShellExecute( Path + Name, NULL, NULL ) == 0 )
		{
			 

		}
	}
//	printf("ahora salio del run\n");
}

bool	MainWindow::SaveAll(bool Query)
{
	Source *s;

	for (sources.last(); sources.current(); sources.prev())
	if ( !sources.current()->main && sources.current()->Modified() )
	{
		s = sources.current();
		
		int action = Query ?  
                        (QMessageBox::warning(this, "Fuente modificada", 
   	       		Format("Desea guardar los cambios hechos en %s?", (s->name+s->ext).data()),
			  3, 4, 2)) : 3; // yes no cancel
				
		// cancel ?
		if ((action==2) || (action==4)) return false; 
		
		// save ?
		if ((action==3) && !saveSource(s)) return false;
	}

	return true;
}

FormDesigner	*MainWindow::findForm(QString Name)
{
	for (forms.first(); forms.current(); forms.next())
	if (Name == forms.current()->name())
	{
		return forms.current();
	}
	return NULL;
}

void MainWindow::ExecTimeOut()
{
	qApp->postEvent(this, new QEvent(QEvent::Timer));
}

