/****************************************************************************
    qtabnote.cpp  13/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include  <qtabnote.h>
#include  <qstrlist.h>

struct QPage: public QTab
{
	QWidget	*page;
	
	QPage(const char *t, QWidget *w)
	: QTab(), page(w)
	{ label = t; }
	~QPage() { if (page) delete page; }
};

QTabNote::QTabNote(QWidget *p)
: QTabBar(p) 
{
	connect(this, SIGNAL(selected(int)), this, SLOT(Selected(int)));
}

QTabNote::~QTabNote()
{ printf("borrando qtabnote\n"); }

void	QTabNote::addTab(QWidget *W, const char *Tab)
{
	if (W) W->hide();
	QTabBar::addTab((QTab*) new QPage(Tab, W));
}

void	QTabNote::resizeEvent(QResizeEvent *e)
{
	if (e) QTabBar::resizeEvent(e);

	QFontMetrics	fm( font() );
	int		U = fm.height() + 10;

	if (tabList()->last() && ((QPage*)tabList()->current())->page)
		((QPage*)tabList()->current())->page->setGeometry(2, U+2, width()-4, height()-U-4);
}

QWidget*  QTabNote::Page(const char *Page)
{
	for (tabList()->first(); tabList()->current(); tabList()->next())
	if (Page && tabList()->current()->label.data() && 
	   (strcasecmp(Page, tabList()->current()->label)==0)) 
	   	return ((QPage*) tabList()->current())->page;
	return NULL;  
}

const 	char *QTabNote::page()
{
	if (tabList()->last()) return tabList()->current()->label;

	return "";
}

void  	QTabNote::setPage(const char *Page)
{
	for (tabList()->first(); tabList()->current(); tabList()->next())
	if (Page && tabList()->current()->label.data() && 
	   (strcasecmp(Page, tabList()->current()->label)==0)) 
	   {
		setCurrentTab(tabList()->current());
		return;
	   }
}

bool	QTabNote::event(QEvent *e)
{

/*
        if ( e->type() == QEvent::ChildInserted )
        {
		QWidget *child = (QWidget*) ((QChildEvent*)e)->child();
		if ( child && child->inherits("QWidget") && 
		     tabList()->last() &&
		     !((QPage*)tabList()->last())->page )
		{
			QFontMetrics	fm( font() );
			int		U = fm.height() + 10;

	 		((QPage*)tabList()->last())->page = child;
			((QPage*)tabList()->last())->page->setGeometry(2, U+2, width()-4, height()-U-4);
		}
		return true;
	}
*/	
	return QTabBar::event(e);
}

void QTabNote::paintEvent(QPaintEvent *e)
{
	
	
    QFontMetrics fm = fontMetrics();
    int		 tabWidth = 2, tabHeight = fm.height() + 10;
    
    for (tabList()->first(); tabList()->current(); tabList()->next())
    tabWidth += ( tabList()->current()->r.width() - 4);

    QTabBar::paintEvent(e);

    QPainter p( this );

    p.setPen( colorGroup().dark());
    p.drawLine( 0, height()-1, width(), height()-1);
    p.drawLine( width()-1, tabHeight, width()-1, height() );
    
    p.setPen( colorGroup().light() );
    p.drawLine( tabWidth, tabHeight-1, width(), tabHeight-1 );
    p.drawLine( 0, tabHeight, 0, height()-1);

}

void	QTabNote::Selected(int page)
{
	QPage *p = (QPage*) QTabBar::tab(page);

	if (p)
	{
		if (p->page)
		{
			QFontMetrics	fm( font() );
			int		U = fm.height() + 10;

			p->page->setGeometry(2, U+2, width()-4, height()-U-4);
			p->page->show();
		}
		
		if (tabList()->last()) 
		{
			p = (QPage*) tabList()->prev();
			if (p && p->page) p->page->hide();
		}
	}
}

static	QStrList	List;

QStrList &QTabNote::pages()
{

	List.clear();
  	if ( tabList() )
    	for (tabList()->first(); tabList()->current(); tabList()->next())
    	{
    	  printf("%s\n", tabList()->current()->label.data());
  	  if (tabList()->current()->label.data()) List.append(tabList()->current()->label.data());
	}
	return List;
}


void	QTabNote::setPages(QStrList &l)
{
	QWidget	*controls[256];
	int i;
	QPage *page;
	QListT<QTab>   List;

    	for (l.first(); l.current(); l.next())
	{
		QTab *p = tab( l.current() );
		
		if (p) 
		{
			List.append( p );
			tabList()->take();
		}
	}
	
	while ( tabList()->first() ) tabList()->remove( tabList()->current() );

 	for (l.first(); l.current(); l.next())
	{
		QTab  *t = NULL;
		
		for (List.first(); List.current(); List.next())
		if (strcasecmp( List.first()->label, l.current())==0) 
		{
			t = List.current();
			List.take();
			break;
		} 
		
		printf("addign %s %lx\n", l.current(), t);
		QTabBar::addTab(t ? t : new QPage(l.current(), NULL));
	}
	repaint(true);	
}

QTab*  QTabNote::tab(const char *Page)
{
	for (tabList()->first(); tabList()->current(); tabList()->next())
	if (Page && tabList()->current()->label.data() && 
	   (strcasecmp(Page, tabList()->current()->label)==0)) 
	   	return tabList()->current();
	return NULL;  
}

