/****************************************************************************
    stdregs.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/


// ================= classes definition ================================
// ======================= rtti ========================================


#include <qtrtti.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <writer.h>
#include <reader.h>
#include <qstrlist.h>

TBinaryDataList  TReader::readers;
TBinaryDataList  TWriter::writers;


//  macros for easy rtti definition

#define  Property(Type, Prop, Set, Kind) new PropInfo(classinfo, &_##Kind, #Prop, &##Type##::##Prop, &##Type##::##Set)

#define  DefineClass(Class, Ancestor, Props)				\
extern   ClassInfo  _##Class; 								\
extern   ClassInfo  _##Ancestor;							\
QWidget	 *create##Class(QWidget *Parent) { return new Class(Parent); }   \
void     props##Class(ClassInfo *classinfo) Props			\
ClassInfo _##Class(#Class, &_##Ancestor, (CreateProc) &create##Class, &props##Class); 

#define  DefineObject(Class, Props)				\
extern   ClassInfo  _##Class; 								\
void     props##Class(ClassInfo *classinfo) Props			\
ClassInfo _##Class(#Class, NULL, NULL, &props##Class); 


/***************************************************************************
                         Builtin  types                                 
***************************************************************************/
OrdinalInfo   	_char("char", 1, 0, 256);
OrdinalInfo	_short("short", 2, -0x7fff, 0x8000);
OrdinalInfo 	_int("int", 4, -0x7fffffff, 0x80000000);
EnumInfo	_bool("bool", 1, "FALSE", 0, "TRUE", 1, NULL);
TypeInfo	_string("string", tkstring);
TypeInfo	_qstring("qstring", tkqstring);
SignalInfo	_signal("signal");


#include <qwindowdefs.h>
extern EnumInfo _Alignment;
EnumInfo	_Alignment("alignment", 4, 
"AlignLeft", Qt::AlignLeft, "AlignRight", Qt::AlignRight, "AlignHCenter", Qt::AlignHCenter, 
"AlignTop", Qt::AlignTop, "AlignBottom", Qt::AlignBottom, "AlignVCenter", Qt::AlignVCenter, 
"SingleLine", Qt::SingleLine,
"DontClip", Qt::DontClip, 
"ExpandTabs", Qt::ExpandTabs,
"ShowPrefix", Qt::ShowPrefix,
"WordBreak", Qt::WordBreak,
"DontPrint", Qt::DontPrint,
NULL);
extern SetInfo _Alignments;
SetInfo	_Alignments(&_Alignment, "alignments");



/***************************************************************************
                         QObject                              
***************************************************************************/
#include <qobject.h>
const char *getName(QObject *obj) { return obj->name(); }
void propsQObject(ClassInfo *classinfo)
{
	new PropInfo(classinfo, &_string, "name", 0xffff0000, &getName, &QObject::setName);
}
ClassInfo _QObject("QObject", NULL, NULL, &propsQObject);




/**************************************************************************
                      QFont
**************************************************************************/
#include <qfont.h>
DefineObject(QFont,  
{
	Property(QFont, family, setFamily, qstring);
	Property(QFont, pointSize, setPointSize, int);
	Property(QFont, weight, setWeight, int);
	Property(QFont, bold, setBold, bool);
	Property(QFont, italic, setItalic, bool);
 	Property(QFont, underline, setUnderline, bool);
	Property(QFont, strikeOut, setStrikeOut, bool);
})

void setRed(QColor &C, int r) { C.setRgb(r, C.green(), C.blue()); }
void setGreen(QColor &C, int g) { C.setRgb(C.red(), g, C.blue()); }
void setBlue(QColor &C, int b) { C.setRgb(C.red(), C.green(), b); }
DefineObject(QColor,  
{
	new PropInfo(classinfo, &_int, "red", &QColor::red, 0xffff0000, &setRed);
	new PropInfo(classinfo, &_int, "green", &QColor::green, 0xffff0000, &setGreen);
	new PropInfo(classinfo, &_int, "blue", &QColor::blue, 0xffff0000, &setBlue);
})


EnumInfo	_PropagationMode("PropagationMode", 1, 
                	"NoChildren", 0, 
	                "AllChildren", 1, 
        	        "SameValue", 2,
                	NULL);




void StoreQStrList(TWriter &w, QObject *o, PropInfo *propinfo)
{
	QStrList *l = (QStrList*) getOrdProp(o, propinfo);
	int  count = l ? l->count() : 0;
	w.write(&count, 4);
	for (int i=0; i<count; i++) w.writeString(l->at(i));
}
void LoadQStrList(TReader &r, QObject *o, PropInfo *propinfo)
{
	QStrList l;
	int  count;
	r.read(&count, 4);
	for (int i=0; i<count;i++) l.append(r.readString());
	setOrdProp(o, propinfo, (unsigned int) &l);
}
DefineObject(QStrList,
{
	TWriter::DefineBinaryProperty("QStrList", "", (void*) &StoreQStrList);
	TReader::DefineBinaryProperty("QStrList", "", (void*) &LoadQStrList);
})
#include <qpixmap.h>

QString loadASCIIFile(QString fileName)
{
	QString  r;
	
	FILE  *f = fopen(fileName, "r");
	if (f) 
	{
		char *tmp;
		long  size;
		fseek(f, 0, SEEK_END);
		size = ftell(f);
		tmp = (char*) malloc(size+1);
		fseek(f, 0, SEEK_SET);
		fread(tmp, 1, size, f);
		fclose(f);
		tmp[size] = 0;
		r = tmp;
		free(tmp);
	} 
	return r;
}
void StoreQPixmap(TWriter &w, QPixmap *p)
{
	QString Data;
	if (p)
	{
		p->save("/tmp/vqt.pixmap.xpm", "XPM");
		Data = loadASCIIFile("/tmp/vqt.pixmap.xpm");
		remove("/tmp/vqt.pixmap.xpm");
	} 
	w.writeString(Data);
}
void storeQPixmap(TWriter &w, QObject *o, PropInfo *propinfo)
{
	StoreQPixmap(w, (QPixmap*) getOrdProp(o, propinfo));
}
QPixmap LoadQPixmap(TReader &r)
{
	QString Data = r.readString();
	if (Data.length()==0) return QPixmap();
	QPixmap  V;
	V.loadFromData((const uchar *) Data.data(), Data.length());
	return V;
}
void loadQPixmap(TReader &r, QObject *o, PropInfo *propinfo)
{
	QPixmap P = LoadQPixmap(r);
	if (!P.isNull()) setOrdProp(o, propinfo, (unsigned int) &P);
}
DefineObject(QPixmap, 
{
	TWriter::DefineBinaryProperty("QPixmap", "", (void*) &storeQPixmap);
	TReader::DefineBinaryProperty("QPixmap", "", (void*) &loadQPixmap);
})




/***************************************************************************
                         QButton                             
***************************************************************************/
#include <qbutton.h>
DefineClass(QButton, QWidget, 
{
	Property(QButton, text, setText, qstring);
})


/***************************************************************************
                         QRadioButton                             
***************************************************************************/
#include <qradiobutton.h>
DefineClass(QRadioButton, QButton, 
{
	Property(QRadioButton, isChecked, setChecked, bool);
})


/***************************************************************************
                         QCheckBox
***************************************************************************/
#include <qcheckbox.h>
DefineClass(QCheckBox, QButton, 
{
	Property(QCheckBox, isChecked, setChecked, bool);
})


/***************************************************************************
                         QDialog
***************************************************************************/
#include <qdialog.h>
void propsQDialog(ClassInfo *classinfo)
{
	Property(QDialog, caption, setCaption, qstring);
}
ClassInfo _QDialog("QDialog", &_QWidget, NULL, &propsQDialog);




/***************************************************************************
                         QPushButton                             
***************************************************************************/
#include <qpushbutton.h>
DefineClass(QPushButton, QButton, 
{
	Property(QButton, pixmap, setPixmap, QPixmap);
})


/***************************************************************************
                         QLineEdit                             
***************************************************************************/
#include <qlineedit.h>
DefineClass(QLineEdit, QWidget, 
{
	Property(QLineEdit, text, setText, qstring);
})






/***************************************************************************
                         QListBox                             
***************************************************************************/
#include <qlistbox.h>
void setLBItems(QListBox *LB, QStrList *L) 
{ 
	LB->clear(); 
	LB->insertStrList(L);
}
QStrList LBHelper;
QStrList &getLBItems(QListBox *LB) 
{ 
	LBHelper.clear(); 
	for (uint i=0; i<LB->count(); i++)
	LBHelper.append(LB->text(i));
	return LBHelper;
}
DefineClass(QListBox, QWidget, 
{
	new PropInfo(classinfo, &_QStrList, "items", 0xffff0000, &getLBItems, 0xffff0000, &setLBItems);
})


/***************************************************************************
                         QComboBox                             
***************************************************************************/
#include <qcombobox.h>
void setCBItems(QComboBox *LB, QStrList *L) 
{ 
	LB->clear(); 
	LB->insertStrList(L);
}
QStrList CBHelper;
QStrList &getCBItems(QComboBox *LB) 
{ 
	CBHelper.clear(); 
	for (int i=0; i<LB->count(); i++)
	CBHelper.append(LB->text(i));
	return CBHelper;
}
DefineClass(QComboBox, QWidget, 
{
	new PropInfo(classinfo, &_QStrList, "items", 0xffff0000, &getCBItems, 0xffff0000, &setCBItems);
})

#include <qmainwindow.h>
DefineClass(QMainWindow, QWidget, 
{
	Property(QMainWindow, caption, setCaption, qstring);
})



#define INCLUDE_MENUITEM_DEF   1
#include <qmenubar.h>
#include <qpopupmenu.h>

class MenuDataHelper: public QMenuData 
{
public:
	QListT<QMenuItem>  *Items() { return (QListT<QMenuItem>  *) mitems; }

};

extern "C" void StoreQMenuItem(TWriter &w, QMenuItem *i);

void StoreQMenuData(TWriter &w, QMenuData *d)
{
	QListT<QMenuItem>  	*items = ((MenuDataHelper*) d)->Items();
	int count = items ? items->count() : 0;
	
	w.write(&count, 4);
	
	if (items)
	for (items->first(); items->current(); items->next())
	{
		StoreQMenuItem(w, items->current());
	}
}
void StoreQMenuItem(TWriter &w, QMenuItem *i)
{
	int type = i->isSeparator() ? 1 : (!i->popup() ? 2 : 3);

	// save type
	w.write(&type, 4);
	if (type == 1) return;

	// save text
	int len  = i->text() ? strlen(i->text()) : 0;
	w.write(&len, 4);
	w.write(len>0 ? (void*) i->text().data() : (void*) &len, len + 1);
	 
	// save the pixmap
	StoreQPixmap(w, i->pixmap());
	
//	checked = i->isChecked(); 		
//	enabled = i->isEnabled(); 		

//	save signal -> slot
	QStrList Slots;
	QSignalList Connections(true);
	getSlots(w.Root, Slots);
	if (getConnections((QObject*) i->signal(), "2x()", w.Root, Slots, Connections))
		w.writeString(Connections.first()->Slot);
	else
		w.writeString("");	

	// save the popup
	if (i->popup()) StoreQMenuData(w, i->popup());

}

void LoadQMenuData(TReader &r, QMenuData *d)
{
	int count;
	
	r.read(&count, 4);

	for (int i=0; i<count; i++)
	{
		int	type;
		r.read(&type, 4);
		if (type == 1) d->insertSeparator(); else
		{
			// load text
			QString text;
			int len;
			r.read(&len, 4);
			char *v = (char*) malloc(len+1);
			r.read(v, len+1);
			text = v;
			delete v;
			
			// load pixmap
			QPixmap  pixmap = LoadQPixmap(r);
	
	
			// checked = i->isChecked(); 		
			// enabled = i->isEnabled(); 		
			// cargar signal -> slot
			QString	Slot = r.readString();
			r.CheckSlot(Slot);
			
			// load popup
			QPopupMenu *popup = NULL;
			if (type == 3) 
			{
				popup = new QPopupMenu;
				LoadQMenuData(r, popup);
			}
			
			if (type == 2) 
			{
				if (pixmap.isNull()) 
				if (Slot.length()>0) d->insertItem(text, r.Root, "1"+Slot);
				else d->insertItem(text);
				else 
				if (Slot.length()>0) d->insertItem(pixmap, text, r.Root, "1"+Slot);
				else d->insertItem(pixmap, text);

			} else
			{
				if (pixmap.isNull()) d->insertItem(text, popup);
				else d->insertItem(pixmap, text, popup);
			}
			
		}	
	}
}
void LoadQMenuBar(TReader &r, QMenuBar *o, PropInfo *propinfo)
{
	LoadQMenuData(r, o);
}
void StoreQMenuBar(TWriter &w, QMenuBar *o, PropInfo *propinfo)
{
	StoreQMenuData(w, o);
}
DefineClass(QMenuBar, QFrame,
{
	TWriter::DefineBinaryProperty("QMenuBar", "", (void*) &StoreQMenuBar);
	TReader::DefineBinaryProperty("QMenuBar", "", (void*) &LoadQMenuBar);
})
void LoadQPopupMenu(TReader &r, QPopupMenu *o, PropInfo *propinfo)
{
	LoadQMenuData(r, o);
}
void StoreQPopupMenu(TWriter &w, QPopupMenu *o, PropInfo *propinfo)
{
	StoreQMenuData(w, o);
}
DefineClass(QPopupMenu, QFrame,
{
	TWriter::DefineBinaryProperty("QPopupMenu", "", (void*) &StoreQPopupMenu);
	TReader::DefineBinaryProperty("QPopupMenu", "", (void*) &LoadQPopupMenu);
})



#include <qbuttongroup.h>
DefineClass(QGroupBox, QFrame,
{
	Property(QGroupBox, title, setTitle, qstring);
})

DefineClass(QButtonGroup, QGroupBox,
{
})



