/****************************************************************************
    typinfo.cpp  24/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include <qtrtti.h>
#include <stdio.h>
#include <qobject.h>
#include <qmetaobject.h>
#include <qobjectdict.h>
#include <stdarg.h>

#define STREQU(S1, S2) (((S1)==(S2)) || ((S1) && (S2) && !strcmp(S1, S2))) 

ClassInfo	*classlist = 0;


TypeInfo::TypeInfo(const char *nm, TypeKind k)
: name(nm), kind(k) 
{ }

SignalInfo::SignalInfo(const char *nm)
: TypeInfo(nm, tksignal) 
{ }

OrdinalInfo::OrdinalInfo(const char *nm, int n, int x, int s)
: TypeInfo(nm, tkord), size(s), min(n), max(x) 
{ }

EnumPair::EnumPair(EnumInfo &info, char *nm, int v)
: name(nm), value(v) 
{ next = info.names; info.names = this; }

ClassInfo::ClassInfo(const char *nm, ClassInfo *bc, CreateProc c, DefineProps p)
: TypeInfo(nm, tkclass), props(0), create(c), ancestor(bc)
{	
	next = classlist;
	classlist = this;
	printf("Class %s", name);
	if (p) p(this);
	printf(" registered (%lx).\n", (unsigned long) this);
}	

PropInfo* ClassInfo::Prop(const char *Name)
{
	for (PropInfo *tmp=props; tmp; tmp=tmp->next)
	{
		if (STREQU(Name, tmp->name)) return tmp;
	}
	return (ancestor ? ancestor->Prop(Name) : NULL);
}

bool ClassInfo::inherits(const char* Name)
{	
	for (ClassInfo *tmp=this; tmp; tmp=tmp->ancestor)
	if (STREQU(Name, tmp->name)) return tmp;

	return NULL;
}

ClassInfo* ClassInfo::Find(const char* Name)
{	
	for (ClassInfo *tmp=classlist; tmp; tmp=tmp->next)
	{
		if (STREQU(Name, tmp->name)) 
		{
			return tmp;
		}
	}
	return NULL;
}

ClassInfo *ClassInfo::Find(QObject *o)
{
	if (o) return Find(o->className());
	else return NULL;
}

PropInfo::PropInfo(ClassInfo *c, TypeInfo *t, char *nm, ...)
: owner(c), name(nm), type(t)
{	
	va_list	args;
	unsigned long l;
	
	next = c->props;
	c->props = this;
	
	va_start(args, nm);
	l = va_arg(args, unsigned long);
	if (l==0xffff0000)               	// Static Method
	{ getf = 0x00000000;
	  getd = va_arg(args, unsigned long);
	} else if (l & 0xffff0000) 		// Virtual Method
	{ getf = 0x40000000 + va_arg(args, unsigned long);
	  getd = (l >> 16) -1;
	} else 					// Field
	{ getf = 0x80000000;
	  getd = l - 1;
	}

	l = va_arg(args, unsigned long);
	if (l==0xffff0000)               	// Static Method
	{ setf = 0x00000000;
	  setd = va_arg(args, unsigned long);
	} else if (l & 0xffff0000) 		// Virtual Method
	{ setf = 0x40000000 + va_arg(args, unsigned long);
	  setd = (l >> 16) -1;
	} else 					// Field
	{ setf = 0x80000000;
	  setd = l - 1;
	}
	
	va_end(args);
	printf(" %s", name);
} 


EnumInfo::EnumInfo(const char * nm, int Size, ...)
: OrdinalInfo(nm, 0, 0, Size)
{	va_list	args;
	char	*iname;
	int	ivalue;

	kind = tkenum;
	names = 0;	
	va_start(args, Size);
	do
	{
		iname = va_arg(args, char*);
		ivalue = va_arg(args, int);
		if (iname && *iname) 
			new EnumPair(*this, iname, ivalue);
	} while (iname && *iname);

	va_end(args);
}

SetInfo::SetInfo(EnumInfo *b, char *nm)
: OrdinalInfo(nm, 0, 0, b->size), base(b)
{
	kind = tkset;
}



typedef union 
{
	void*	*vmt;
	char	data[256];
} *AnyObj;

#define OBJ ((AnyObj) obj)
#define STATIC(f) propinfo->##f##d
#define VIRTUAL(F) ((AnyObj) (((char*) obj)+(propinfo->##F##f & 0x0000ffff)))->vmt[propinfo->##F##d]
#define FIELD(F) (&((AnyObj) obj)->data[propinfo->##F##d])

unsigned long getOrdProp(void* obj, PropInfo* propinfo)
{
	unsigned long value = 0;
	
	int  Size = (propinfo->type->kind == tkclass) ? 32 : 8 * ((OrdinalInfo*) propinfo->type)->size;

	switch (propinfo->getf & 0xc0000000) {
	case 0x00000000: 
		value = ((unsigned long (*)(void*)) STATIC(get)) (obj); 
		break;
	case 0x40000000: 
		value = ((unsigned long (*)(void*)) VIRTUAL(get)) (obj); 
		break;
	case 0xc0000000: 
		value = *((unsigned long *) FIELD(get)); 
		break;
	}
	return value & (0xFFFFFFFF >> (32-Size));
}

void setOrdProp(void* obj, PropInfo* propinfo, unsigned long value)
{
	int  Size = (propinfo->type->kind == tkclass) ? 32 : 8 * ((OrdinalInfo*) propinfo->type)->size;
	value &= (0xFFFFFFFF >> (32-Size));

	switch (propinfo->setf & 0xc0000000) {
	case 0x00000000: 
		((void (*)(void*, unsigned long)) STATIC(set)) (obj, value); 
		break;
	case 0x40000000: 
		((void (*)(void*, unsigned long)) VIRTUAL(set)) (obj, value); 
		break;
	case 0xc0000000: 
		*((unsigned long *) FIELD(set)) = value; 
		break;
	}
}


QString  getStrProp(void* obj, PropInfo* propinfo)
{
	QString value;

	if (propinfo->type->kind == tkstring)
	switch (propinfo->getf & 0xc0000000) {
	case 0x00000000: 
		value = ((char* (*)(void*)) STATIC(get)) (obj); 
		break;
	case 0x40000000: 
		value = ((char* (*)(void*)) VIRTUAL(get)) (obj); 
		break;
	case 0xc0000000: 
		value = *((char* *) FIELD(get)); 
		break;
	} else 
	if (propinfo->type->kind == tkqstring)
	switch (propinfo->getf & 0xc0000000) {
	case 0x00000000: 
		value = ((QString (*)(void*)) STATIC(get)) (obj); 
		break;
	case 0x40000000: 
		value = ((QString (*)(void*)) VIRTUAL(get)) (obj); 
		break;
	case 0xc0000000: 
		value = *((QString *) FIELD(get)); 
		break;
	}
	return value;
}

void setStrProp(void* obj, PropInfo* propinfo, const char* value)
{
	QString qvalue = value;
	
	if (propinfo->type->kind == tkstring)
	switch (propinfo->setf & 0xc0000000) {
	case 0x00000000: 
		((void (*)(void*, const char*)) STATIC(set)) (obj, value); 
		break;
	case 0x40000000: 
		((void (*)(void*, const char*)) VIRTUAL(set)) (obj, value); 
		break;
	case 0xc0000000: 
		*((const char **) FIELD(set)) = value; 
		break;
	} else
	if (propinfo->type->kind == tkqstring)
	switch (propinfo->setf & 0xc0000000) {
	case 0x00000000: 
		((void (*)(void*, QString&)) STATIC(set)) (obj, qvalue); 
		break;
	case 0x40000000: 
		((void (*)(void*, QString&)) VIRTUAL(set)) (obj, qvalue); 
		break;
	case 0xc0000000: 
		*((QString*) FIELD(set)) = qvalue; 
		break;
	}
}


QSignalProp getSignalProp(void* obj, PropInfo* propinfo)
{
	QSignalProp  value;
	
	if (!obj || !propinfo) return value;
	
	switch (propinfo->getf & 0xc0000000) {
	case 0x00000000: 
		value = ((QSignalProp (*)(void*)) STATIC(get)) (obj); 
		break;
	case 0x40000000: 
		value = ((QSignalProp (*)(void*)) VIRTUAL(get)) (obj); 
		break;
	case 0xc0000000: 
		value = *((QSignalProp *) FIELD(get)); 
		break;
	}
	return value;
}

void setSignalProp(void* obj, PropInfo* propinfo, QSignalProp value)
{
	if (!obj || !propinfo) return;

	switch (propinfo->setf & 0xc0000000) {
	case 0x00000000: 
		((void (*)(void*, QSignalProp)) STATIC(set)) (obj, value); 
		break;
	case 0x40000000: 
		((void (*)(void*, QSignalProp)) VIRTUAL(set)) (obj, value); 
		break;
	case 0xc0000000: 
		*((QSignalProp *) FIELD(set)) = value; 
		break;
	}
}




void createClassInfo(QObject *o)
{
	if ( ClassInfo::Find(o) ) return ;
	
	new ClassInfo(o->className(), ClassInfo::Ancestor(o), NULL, NULL);
}


ClassInfo *ClassInfo::Ancestor(QObject *o)
{
	ClassInfo *classinfo;
	if (!o) return NULL;

	if ( (classinfo = ClassInfo::Find( o )) ) return classinfo;
	
	for (QMetaObject *meta=o->metaObject(); meta; meta = meta->superClass())
	if ( (classinfo = ClassInfo::Find(meta->className()))) return classinfo;
	
	return NULL;
}

ClassInfo *ClassInfo::Ancestor(const char *Name)
{
	if (!Name || !*Name) return NULL;
	
	ClassInfo *classinfo = ClassInfo::Find(Name);

	if (classinfo) return classinfo;

	if (objectDict)
	for (QMetaObject *meta=objectDict->find(Name); meta; meta = meta->superClass())
	if ( (classinfo = ClassInfo::Find(meta->className()))) return classinfo;
	
	return NULL;
}


class QObjectHelper: public QObject {
public:
	QConnectionList *receivers(const char *s) { return QObject::receivers(s); }
};


bool getConnections(QObject *o, QString Signal, QObject *receiver,
                       QStrList Slots, QSignalList &Connections)
{
	if ( !o || !receiver ) return false;

	QListT<QConnection> *l = (QListT<QConnection>*) ((QObjectHelper*) o)->receivers(Signal);
	
	Connections.clear();
	
	if ( !l ) return false;
	
	for(l->first(); l->current(); l->next())
	if (
		l->current()->object() == receiver &&
		Slots.find(l->current()->memberName()) != -1
	   )
	{
		Connections.append(
			new QSignalProp(
				l->current()->object(),
				l->current()->memberName()
			)
		);
	}
	
	return Connections.count() > 0;
}


bool getSignals(QObject *o, QStrList &Signals)
{	
	Signals.clear();

	if ( o ) Signals = o->metaObject()->signalNames(TRUE);
	
	return Signals.count() > 0;
}

bool getSlots(QObject *o, QStrList &Slots)
{
	Slots.clear();
	
	if ( o ) Slots = o->metaObject()->slotNames( true );
	
	return Slots.count() > 0;
}

