/****************************************************************************
    writer.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include <writer.h>
#include <qobject.h>
#include <string.h>
#include <stdio.h>
#include <qlist.h>
#include <qconnection.h>
#include <qmetaobject.h>
#include <qapplication.h>

void  TWriter::writePropPath(PropInfo *propinfo)
{
	if (propPath.length()>0) writeString(propPath + propinfo->name);
	else writeString(propinfo->name);
}

void  TWriter::writeInteger(unsigned long v)
{
   writeValue(vaInt);
   write(&v, 4);
}

void  TWriter::writeString(const char *v)
{
   int  l = (v ? strlen(v) : 0);
   writeValue(vaStr);
   write(&l, 4);
   if (l) write(&v[0], l+1);
}

void  TWriter::writeIdent(const char *v)
{
  writeString(v);
}

void  TWriter::writeValue(TValueType v)
{
  write(&v, 1); // no se si en todos los hards se puede, supongo que si 
}

void  TWriter::writeProperty(QObject *o, ClassInfo *classinfo, PropInfo *propinfo)
{
	// ver si hay registrado algun writer para la propiedad
	void *Writer = writers.mapper(classinfo->name, propinfo->name);
	
	if (Writer)
	{
		writePropPath(propinfo);
		((WriterProc) Writer)(*this, o, propinfo);
		return;	
	}

	// ver si hay registrado algun writer para la clase
	if (propinfo->type->kind == tkclass) Writer = writers.mapper(propinfo->type->name, "");
	if (Writer) 
	{
		writePropPath(propinfo);
		((WriterProc) Writer)(*this, o, propinfo);
		return;
	}
	switch (propinfo->type->kind) {
	case tkundef: 
//		fix this
	
		break; 
	case tkset:
	case tkenum:
	case tkord: {
		unsigned long value = getOrdProp(o, propinfo);
		writePropPath(propinfo);
		writeInteger(value);
		} break;
	case tkqstring:
	case tkstring: {
		QString value = getStrProp(o, propinfo);
		if (value.length()==0) break;
		writePropPath(propinfo);
		writeString(value);
		} break;
	case tkclass: {

/* alguna explicacion debe ir aqui por ser diferente 

Una propiedad de este tipo puede ser:
1. una clase que herede a QObject, en este caso solo se guarda el nombre porque
   las propiedades del objeto en cuestion se guardaran independientemente

2. otra clase cualquiera. se desglozan las propiedades y se guardan utilizando 
   propPath.

*/
		unsigned long value = getOrdProp(o, propinfo);
		if (!value) break;
		 
		if (((ClassInfo*)propinfo->type)->inherits("QObject"))
		{
			writePropPath(propinfo);
			writeIdent( ((QObject*) value)->name() );
		} else
		{	
			QObject	*saveObj = o;
			QString saveProp = propPath;
			
			if (propPath.length()>0) propPath = (propPath + propinfo->name) + ".";
			else propPath = QString(propinfo->name) + ".";
			
			o = (QObject*) value;
			writeProperties( o, (ClassInfo*) propinfo->type);
			o = saveObj;

			propPath = saveProp; 
		}

		} break;			 
	default:
		break;
	}
}

void  TWriter::writeProperties(QObject *o, ClassInfo *classinfo)
{	
	for (ClassInfo *cls = classinfo; cls; cls=cls->ancestor)
	for (PropInfo *tmp=cls->props; tmp; tmp=tmp->next)
	{
		writeProperty(o, cls, tmp);
	}
}


void WritePrivateObject(TWriter &w, QObject *o, void *Writer, ClassInfo *classinfo)
{
	QGetPosEvent 	event(1);
	qApp->sendEvent(o, &event);
	w.writeString(o->name());
	w.writeInteger(event.X);
	w.writeInteger(event.Y);
	((WriterProc) Writer)(w, o, classinfo);
}

void  TWriter::writeObject(QObject *o)
{	
	Object = o;
	if ( Root == NULL) Root = o;

	ClassInfo *classinfo = ClassInfo::Find(o);

	if (!classinfo) return;

	writeString(o->className());
	int iniPos = at();
	write(&iniPos, 4);

	void *Writer = writers.mapper(classinfo->name, "");

	if (Writer) WritePrivateObject(*this, o, Writer, classinfo);
	else
	{
	
		writeProperties(o, classinfo);
		writeString("");
	
		writeSignals(o);
		writeString("");

		writeChildren(o);
		writeString("");
	}

	int endPos = at();
	int Size = endPos - iniPos - 4;
	
	at(iniPos);
	write(&Size, 4);
	at(endPos);
}


void  TWriter::writeChildren(QObject *o)
{	
	if (o->children()) 
	{
		QListT<QObject> *l = (QListT<QObject>*) o->children();
		for (l->first(); l->current(); l->next() )
		if ( stored(l->current()) )
  			writeObject(l->current());
	}
}

class QSignalWriter: public QObject {
public:
	QConnectionList *receivers(const char *s) { return QObject::receivers(s); }
};

void  writeSignal(TWriter *w, QSignalWriter *o, const char *signal)
{
	QListT<QConnection> *l = (QListT<QConnection>*) o->receivers(signal);
	
	if ( l ) 
	for(l->first(); l->current(); l->next())
	if (
		l->current()->object() == w->Root
//		&& es un slot de el
	   )
	{

		w->writeString(signal);
  		w->writeString(l->current()->object()->name());
		w->writeString(l->current()->memberName());

	}
}

void  writeRootSignals(TWriter *w, QObject *o, QMetaObject *meta)
{
	if (!w || !o || !meta) return;

	for (int i=0; i<meta->numSignals(); i++)
	{	
		writeSignal(w, (QSignalWriter*) o, meta->signal(i)->name);
	}
	writeRootSignals(w, o, meta->superClass());
}

void TWriter::writeSignals(QObject *o)
{	
	writeRootSignals(this, o, o->metaObject());
}


void WriteObjectTemplate(QString File, QObject *Obj)
{	
	TWriter  writer(File);
	writer.writeObject(Obj);
}

bool  TWriter::stored(QObject *o)
{
	return o->name() && *o->name() && ClassInfo::Find(o);
}
