/****************************************************************************
    writer.h  11/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#ifndef _writer_
#define _writer_

#include <qtrtti.h>
#include <qfile.h>
#include <qstring.h>
#include <qlist.h>

typedef enum { vaNull, vaInt, vaStr, vaFloat, vaList,
               vaBinary } TValueType;

class    TBinaryData
{
public:
	QString		classname;
	QString		propname;
	void		*filer;
	TBinaryData(const char *c, const char *p, void *f)
	: classname(c), propname(p), filer(f) {}
};
class TBinaryDataList: public QListT<TBinaryData> {
public:
	void *mapper(QString c, QString p)
	{
		for ( first(); current(); next() )
		if (current()->classname == c && current()->propname == p)
			return current()->filer;
		return NULL;
	}	
};

class TWriter: public QFile {
        QString  propPath;
public:
	QObject	*Object;
	QObject	*Root;
static  TBinaryDataList		writers;
static  void DefineBinaryProperty(QString c, QString n, void *f)
	{ writers.append(new TBinaryData(c, n, f)); }
	TWriter(QString Name): QFile(Name), propPath(0), Root(NULL) { open(IO_WriteOnly); }

	void  write(const void *v, unsigned int l) { writeBlock((const char *) v, l); }

	void  writeObject(QObject *o);
	void  writeSignals(QObject *o);
	void  writeChildren(QObject *o);
	void  writeProperties(QObject *o, ClassInfo *);
	void  writeProperty(QObject *o, ClassInfo *, PropInfo *propinfo);
	void  writePropPath(PropInfo *propinfo);
	void  writeInteger(unsigned long v);
	void  writeString(const char* v);
	void  writeIdent(const char * v);
	void  writeValue(TValueType v);
	virtual bool  stored(QObject *);
};
typedef  void  (*WriterProc)(TWriter &, void *, void *); 


#endif


