/****************************************************************************
    srcedit.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "srcedit.h"
#include "editor.h"
#include "../mainwnd.h"
#include <qtabbar.h>
#include <qlistbox.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qtrtti.h>

struct QSource: public QTab {
	QString	&text;
	bool	&modified;
	int	x, y;
	QSource(QString Name, QString &Text, bool &Modified)
	: QTab(), text(Text), modified(Modified), x(0), y(0)
	{ 
		enabled = true;
		label = Name;
	}
	virtual ~QSource() 
	{ 
	} 
};


class  QPages: public QTabBar {
public:
	QPages(QWidget*p): QTabBar(p) {}
	QListT<QSource>	*tabs() 
	{ return (QListT<QSource>*) QTabBar::tabList(); }

};

SourceEditor 	*sourceEditor = NULL;
QLabel		*lCursor;
QLabel		*lModified;


SourceEditor::SourceEditor()
: QMainWindow(NULL, "srcEditor"), source(NULL)
{
	sourceEditor = this;

	setGeometry(200, 100, 500, 400);
	setCaption("Source Editor");

	pages  = new QPages(this);
	QFontMetrics fm = fontMetrics();
	pages->setGeometry(0, 0, width(), fm.height() + 11);
	connect(pages, SIGNAL(selected(int)), this, SLOT(Selected(int)));

	editor = new TSrcEdit(this);

	messages = new QListBox(this);
	messages->setGeometry(0, 0, 10, 1);

	status = new QStatusBar(this);

	status->addWidget(lCursor = new QLabel("1:1", status, "", QFrame::Panel | QFrame::Sunken), 100, true);
	status->addWidget(lModified = new QLabel("", status, "", QFrame::Panel | QFrame::Sunken), 100, true);
	status->addWidget(new QLabel("Insert", status, "", QFrame::Panel | QFrame::Sunken), 100, true);

}


void SourceEditor::resizeEvent(QResizeEvent *e)
{
	QMainWindow::resizeEvent(e);
	realign();
}

void SourceEditor::realign()
{
	int tmp = height();
	
	status->setGeometry(0, tmp-status->height(), width(), status->height());
	tmp-= status->height();
	messages->setGeometry(0, tmp-messages->height(), width(), messages->height());
	tmp-= messages->height();
	pages->setGeometry(0, 0, width(), pages->height());
	tmp-= pages->height();
	editor->setGeometry(0, pages->height(), width(), tmp);

}

void  SourceEditor::Selected(int page)
{
	if (source)
	{
		if (page == source->id) return;
		if (modified) source->modified = true;
		source->text = editor->text();
		editor->getCursorPosition(&source->x, &source->y);
	}	

	source = (QSource *) pages->tab(page);

	if (source)
	{
		editor->setText(source->text);
		modified = source->modified;
		editor->setCursorPosition(source->x, source->y);
		editor->CursorChanged(source->x, source->y);
	} else
	{
		editor->setText("");
		modified = false;
	}
	lModified->setText(modified ? "Modified": "");
}

void  SourceEditor::UpdateSource(QSource *edit)
{
	if ( source && (source==edit) )
	{
		editor->setText(source->text);
		modified = source->modified;
		lModified->setText(modified ? "Modified": "");
	} 
}




QSource	*SourceEditor::NewPage(QString Name, QString &Text, bool &Modified)
{
	QTab	*tab = new QSource(Name, Text, Modified);
	pages->addTab(tab);
	Selected(tab->id);
	pages->setGeometry(0, 0, width()-1, pages->height()); 
	pages->setGeometry(0, 0, width(), pages->height()); 
	
	show();
	return (QSource*) tab;
}

void  SourceEditor::UpdateContents(QSource *edit)
{
	if ( modified && source && (source == edit) ) 
	{
		source->modified = true;
		source->text = editor->text();
		modified = false;
	}
}

void  SourceEditor::ClosePage(QSource *s)
{
	if (!s) return;
	
	if (s==source) 
	{
		UpdateContents(s);
		source = NULL;
	}
	
	QListT<QSource> *l = ((QPages*)pages)->tabs();

	l->remove(s);

	Realign();
	
	if ( !l->last() ) 
	{
		hide();
		editor->setText("");
		modified = false;
		return;
	}
	
	if (!source)
	{
		pages->setCurrentTab( l->last()->id );
		Selected(l->last()->id);
	}
}

bool	SourceEditor::queryClose()
{
	hide();
	return false;
}

void    SourceEditor::Realign()
{
	int i = 0;
	
	QListT<QSource> *l = ((QPages*)pages)->tabs();
	QSource *tab;

	for ( l->first(); l->current(); l->next() )
	if (l->current()->id+1>i) i = l->current()->id+1;
	
    	QFontMetrics fm = fontMetrics();
    	int	p = 0;
	for (int j = 0; j<i; j++)
	{
		tab = (QSource*) pages->tab(j);
		if (tab)
		{
    			int lw = fm.width( tab->label );
			tab->r.setRect( p, 0, lw + 24, pages->height());
			p += (lw + 24 - 3);
		}
	}
	pages->setGeometry(0, 0, width()-1, pages->height()); 
	pages->setGeometry(0, 0, width(), pages->height()); 
}
