/****************************************************************************
    srcmaker.cpp  30/Jun/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include "files.h"
#include "mainwnd.h"
#include "srcmaker.h"
#include "strutils.h"

extern QString exePath;

/***********************************************************************
  Esta funcion es llamada cuando se crea una ventana, para generar 
  las fuentes asociadas
  IN:
  	Window    : Ventana creada
  	Name      : nombre del fichero temporal
  	cpp e inc : son las fuentes donde asociadas a Window
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
**************************************************************************/

int	srcCreateWindow(QMainWindow *Window, QString Name, QString &cpp, QString &inc)
{
	int  pos, len;
	
	cpp = loadASCIIFile(exePath + "../template/window.cpp");
	inc = loadASCIIFile(exePath + "../template/window.h");
	
	pos = 0;
	while ( FindStr(pos, len, inc, "NAME") )
	{
		ReplaceStr(pos, len, inc, Window->name());
		pos += strlen(Window->name());
	}
	pos = 0;
	while ( FindStr(pos, len, cpp, "FILE") )
	{
		ReplaceStr(pos, len, cpp, Name);
		pos += Name.length();
	}
	pos = 0;
	while ( FindStr(pos, len, cpp, "NAME") )
	{
		ReplaceStr(pos, len, cpp, Window->name());
		pos += strlen(Window->name());
	}
	return 3;
}

/***********************************************************************
  Esta funcion es llamada cuando se renombra un fichero que tiene una ventana
  IN:
  	Window    : Ventana 
  	Name 	  : Nombre del fichero
  	NewName   : nombre nuevo del fichero 
  	cpp e inc : son las fuentes donde asociadas a Window
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
**************************************************************************/

int	srcSaveAsWindow(QMainWindow *Window, QString Name, QString NewName, QString &cpp, QString &inc)
{
	int  pos, len;
	
	pos = 0;
	while ( FindStr(pos, len, inc, "_vqt_%s_", Name.data()) )
	{
		ReplaceStr(pos, len, inc, "_vqt_%s_", NewName.data());
		pos += NewName.length();
	}
	pos = 0;
	while ( FindStr(pos, len, cpp, "%s.h", Name.data()) )
	{
		ReplaceStr(pos, len, cpp, "%s.h", NewName.data());
		pos += NewName.length();
	}
	return 3;
}



/***********************************************************************
  Esta funcion es llamada cada vez que se adiciona un componente a una ventana
  IN:
  	Window    : Ventana editada donde se adiciono el componente
  	Parent    : Widget donde se adiciono (puede ser igual a Window)
  	Child     : Componente que se adiciono.
  	cpp e inc : son las fuentes donde asociadas a Window
  	incFile   : el include donde se define la clase
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
----------------------------------------------------
version 1.0	Ernesto Artaza 	

	se define la clase del Child en inc:
	class  ChildClass;

	se define el Child dentro de la definicion de la ventana
	ChildClass	*ChildName;

	no se modifica la fuente cpp
***********************************************************************/
int	srcAddObject(QMainWindow *Window, QObject *Child, QString &cpp, QString &inc, const char *incFile)
{
	int len, pos, r = 0;

	// en caso de no estar definida ya la clase del componente
	// se define
	pos = 0;
	if ( !FindStr(pos, len, inc, "#include <%s>", incFile) )
	{
	 	pos = 0;
	 	if (FindStr(pos, len, inc, "#include <qmainwindow.h>[^\\n]*\\n"))
	 	{
	 		ReplaceStr(pos+len, 0, inc, "#include <%s>\n", incFile );
	 		r = 1;
	 	}
	 } 
	 
	 // se define el componente dentro de la clase de la ventana
	pos = 0;
	if ( FindStr(pos, len, inc, "\\n[\\t\\s]*%s[\\t\\s]*(QWidget[\\t\\s]*\\*[\\t\\s]*)[\\t\\s]*;", Window->name()) )
	{
 		ReplaceStr(pos, 0, inc, "\n    %-16s *%s;", Child->className(), Child->name());
 		r = 1;
	} 
	return r;
}


/***********************************************************************
  Esta funcion es llamada cada vez que se renombra un componente
  IN:
  	Window    : Ventana editada donde se adiciono el componente
  	Child     : Componente que se renombra (aun no tiene el nombre nuevo)
  	Name	  : Nombre que se le va a poner
  	cpp e inc : son las fuentes donde asociadas a Window
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
----------------------------------------------------
version 1.0	Ernesto Artaza 	

	se cambia la definicion del objeto dentro de la clase "ventana"
	ChildClass	*ChildName; => ChildClass   *NewChildName;
	no se modifica la fuente cpp
***********************************************************************/
int	srcRenameObject(QMainWindow *Window, QObject *Child, QString Name, QString &cpp, QString &inc)
{
	int len, pos;

	pos = 0;
	if ( FindStr(pos, len, inc, "%s[\\s\\t]*\\*[\\s\\t]*%s[\\s\\t]*;", Child->className(), Child->name()) )
	{
		ReplaceStr(pos, len, inc, "%s\t*%s;", Child->className(), Name.data());
		return 1;
	}
	return 0;
}

/***********************************************************************
  Esta funcion es llamada cada vez que se renombra una ventana
  IN:
  	Window    : Ventana editada donde se adiciono el componente
  	Name	  : Nombre que se le va a poner
  	cpp e inc : son las fuentes donde asociadas a Window
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
----------------------------------------------------
version 1.0	Ernesto Artaza 	

	se cambia la definicion del objeto dentro de la clase "ventana"
	ChildClass	*ChildName; => ChildClass   *NewChildName;
	no se modifica la fuente cpp
***********************************************************************/
int	srcRenameWindow(QMainWindow *Window, QString Name, QString &cpp, QString &inc)
{

	ReplaceAllStr(inc, 
		Format("_%s_", Window->name()),
		Format("_%s_", Name.data())
	);

	ReplaceAllStr(inc, 
		Format("class %s", Window->name()),
		Format("class %s", Name.data())
	);

	ReplaceAllStr(inc, 
		Format("%s(QWidget", Window->name()),
		Format("%s(QWidget", Name.data())
	);

	ReplaceAllStr(cpp, 
		Format("%s::%s", Window->name(), Window->name()),
		Format("%s::%s", Name.data(), Name.data())
	);

	ReplaceAllStr(cpp, 
		Format("%s::", Window->name()),
		Format("%s::", Name.data())
	);

	return 3;
}

/***********************************************************************
  Esta funcion es llamada cada vez que se borra un componente
  IN:
  	Window    : Ventana editada donde se borra el componente
  	Parent    : Padre del componente que se borra
  	Child     : Componente que se borra
  	cpp e inc : son las fuentes donde asociadas a Window
  OUT:
  	bit 1     : Se modifico la fuente inc
  	bit 2  	  : Se modifico la fuente cpp     
	negativo  : error, no adicionar el componente
----------------------------------------------------
version 1.0	Ernesto Artaza 	

	eliminar la definicion del objeto dentro de la clase "ventana"
	ChildClass	*ChildName;
	no se modifica la fuente cpp
***********************************************************************/
int	srcDeleteObject(QMainWindow *Window, QObject *Child, QString &cpp, QString &inc)
{
	int len, pos;

	pos = 0;
	if ( FindStr(pos, len, inc, "\\n[\\s\\t]*%s[\\s\\t]*\\*[\\s\\t]*%s[\\s\\t]*;[^\\n]*", Child->className(), Child->name()) )
	{
		ReplaceStr(pos, len, inc, "");
		return 1;
	}
	return 0;
}


