/****************************************************************************
    stdregs.cpp  10/Jul/2000
    Copyright (C)  Ernesto F. Artaza  <artaza@info.isctn.edu.cu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
****************************************************************************/

#include <qapp.h>
#include <qfiledialog.h>
#include <qpixmap.h>
#include <qstring.h>
#include <stdlib.h>
#include <qtrtti.h>
#include "formedit/designer.h"

extern QMainWindow *mainWindow;

// ================== componentes ==================================


#include "icons.xpm"

#include <qpushbutton.h>
#include <qmenubar.h>

template <class type> class QObjectHelper: public type {
public:
	Designer	*designer;
	
	QObjectHelper(Designer *P)
	: type(P->designer), designer(P)
	{ initMetaObject(); }
	virtual bool event(QEvent *e)
	{
		if (e->type() == QEvent::User + 1)
		{
			((QGetPosEvent*)e)->X = designer->Widget()->x();
			((QGetPosEvent*)e)->Y = designer->Widget()->y();
			return true;
		}
		if (e->type() == QEvent::User + 2)
		{
			designer->Widget()->setGeometry( ((QGetPosEvent*)e)->X, ((QGetPosEvent*)e)->Y,
			 designer->Widget()->width(), designer->Widget()->height());
			return true;
		}
		return type::event(e);
	}
};


template <class type> class Component1: public Designer, public QPushButton {
private:
	QObjectHelper<type>	  *Object;
public:
	virtual QObject   *Instance() { return Object; }
	virtual QWidget   *Widget() { return this; }      
public:
	Component1(Designer *p, int F)
	: Designer(p ? p->designer : NULL, F), QPushButton(p ? p->designer : NULL)
	{ 
		Object = new QObjectHelper<type>(p);
		
		setPixmap(QPixmap((const char **) ((*componentList)[Instance()->className()]->img)));
		setGeometry(0, 0, 29, 29);
		
		// if (designer && (Flags & cfNoShow) == 0) Object->show();
	}
	virtual bool	defaultHandler(QEvent *e) { return QPushButton::event(e); }
	virtual bool 	event(QEvent *e) { return designer->isDesignerEvent(this, e) 
	                                   || QPushButton::event(e); }
};




RegisterComponent(MenuBar, "Qt", "qmenubar.h", QMenuBar, cfNoSize | cfNoMove | cfNoChild);

Designer *designerCreateQPopupMenu(Designer *p) 
{ return new Component1<QPopupMenu>(p, cfNoSize | cfNoChild | cfNoShow); }	
static ComponentClass classQPopupMenu(PopupMenu, "Qt", "qpopupmenu.h", "QPopupMenu", designerCreateQPopupMenu);




#include <qlabel.h>
RegisterComponent(Label, "Qt", "qlabel.h", QLabel, 0);

#include <qlineedit.h>
RegisterComponent(LineEdit, "Qt", "qlineedit.h", QLineEdit, 0);

#include <qmultilinedit.h>
RegisterComponent(MultiLineEdit, "Qt", "qmultilinedit.h", QMultiLineEdit, 0);

#include <qpushbutton.h>
RegisterComponent(PushButton, "Qt", "qpushbutton.h", QPushButton, 0);

#include <qlistbox.h>
RegisterComponent(ListBox, "Qt", "qlistbox.h", QListBox, 0);

#include <qcombobox.h>
RegisterComponent(ComboBox, "Qt", "qcombobox.h", QComboBox, 0);

#include <qradiobutton.h>
RegisterComponent(RadioButton, "Qt", "qradiobutton.h", QRadioButton, 0);

#include <qcheckbox.h>
RegisterComponent(CheckBox, "Qt", "qcheckbox.h", QCheckBox, 0);

#include <qgroupbox.h>
RegisterComponent(GroupBox, "Qt", "qgroupbox.h", QGroupBox, 0);

#include <qbuttongroup.h>
RegisterComponent(ButtonGroup, "Qt", "qbuttongroup.h", QButtonGroup, 0);


#include <qtabnote.h>
#include "tabbar.xpm"
RegisterComponent(TabBar, "vqt", "qtabnote.h", QTabNote, 0);


#include <qscrollbar.h>
#include "scroll.xpm"
Designer *designerCreateQVScrollBar(Designer *p) 
{ 
	Component<QScrollBar> *scroll = new Component<QScrollBar>(p, 0);
	scroll->setOrientation(QScrollBar::Vertical);
	scroll->setGeometry(0, 0, 20, 100);
	return scroll; 
}	
Designer *designerCreateQHScrollBar(Designer *p) 
{ 
	Component<QScrollBar> *scroll = new Component<QScrollBar>(p, 0);
	scroll->setOrientation(QScrollBar::Horizontal);
	scroll->setGeometry(0, 0, 100, 20);
	return scroll; 
}	
static ComponentClass classQVScrollBar(VScrollBar, "Qt", "qscrollbar.h", "QVScrollBar", &designerCreateQVScrollBar);
static ComponentClass classQHScrollBar(HScrollBar, "Qt", "qscrollbar.h", "QHScrollBar", &designerCreateQHScrollBar);

/*
#include <qscrollview.h>

class QScrollViewComponent: public Component<QScrollView>
{
public:
	QScrollViewComponent(Designer *p): Component(p, 0) { }
	bool eventFilter(QObject*, QEvent *);
	bool event(QEvent *);
;

bool QScrollViewComponent::event(QEvent *e)
{

        if ( e->type() == QEvent::ChildInserted )
        {
		QWidget *child = (QWidget*) ((QChildEvent*)e)->child();
		if ( child && viewport() && child != viewport() &&
		     child != cornerWidget() && child != horizontalScrollBar() &&
		     child != verticalScrollBar()  )
		{
			child->recreate( viewport(), 0, child->geometry().topLeft(), true );	
			removeChild( child );
			viewport()->insertChild( child );
			addChild( child, child->x(), child->y() );
		}
		return true;
	}
	return Component::event(e);

}

Designer *designerCreateQScrollView(Designer *p)
{ return new QScrollViewComponent(p); }	
static ComponentClass classQScrollView(ListBox, "Qt", "qscrollview.h", "QScrollView", designerCreateQScrollView);


bool QScrollViewComponent::eventFilter( QObject *obj, QEvent *e )
{
    QScrollView::eventFilter(obj, e);
    	
    if ( obj == viewport() )
	switch ( e->type() ) 
	{
	  case QEvent::Paint:
	    PaintWidget(viewport(), width(), height(), true);
	    return true;
	  case QEvent::MouseButtonPress:
	  case QEvent::MouseButtonRelease:
	  case QEvent::MouseButtonDblClick:
	  case QEvent::MouseMove:
	    qApp->sendEvent(this, e);
	    return true;
	  default:
	    break;
	}
    return false;
}
*/



/**************************************************************************/

#include "inspector/inspector.h"
#include "inspector/dsgnintf.h"
#include "inspector/dsgnedit.h"

/************************************************************************
	   QObject::name  property editor
***********************************************************************/	   
class TNameProperty : public TStringProperty
{
public:
	virtual int GetAttributes() { return 0; }
	virtual void SetValue(const char *Value)
	{ 
		QWidget	*Top = objectInspector->designer;
	
		if (!Value || !*Value || FindChild(Top, Value))
		{
			QMessageBox::critical(Top,
			"Error!", "No valid name", 0x101, 0, 0);
	
		} else 
		{
			objectInspector->designer->RenameComponent(GetValue(), Value);
			TStringProperty::SetValue(Value);
//			objectInspector->getComponentList(inspector->topObject, "");
		}
	}
};

PropEditorClass<TNameProperty> _TNameProperty("Name");
RegisterPropertyEditor(QObject, name, TNameProperty);




/*************************************************************************
		QFont class editor
*************************************************************************/
#include <qfontdialog.h>
class TFontProperty : public TClassProperty
{
public:
	virtual int GetAttributes() { return paMultiSelect | paSubProperties | paDialog; }
	virtual int Edit()
	{

                bool   ok;
		QFont  newFont = *((QFont*) getOrdProp(aInst(0), aProp(0)));
		
		newFont = QFontDialog::getFont(&ok, newFont, mainWindow);
		
		if (ok) SetOrdValue((unsigned int) &newFont);

		return 0;
 	}
 };

PropEditorClass<TFontProperty> _TFontProperty("Font");
RegisterClassEditor(QFont, TFontProperty);



/**********************************************************************
                    QStrList  class editor
**********************************************************************/
#include "dlgs/stringsdlg.h"
class TStringsProperty : public TClassProperty
{
public:
	virtual int GetAttributes() { return paMultiSelect | paDialog; }
	virtual int Edit()
	{
		QStrList  *Strings = (QStrList*) getOrdProp(aInst(0), aProp(0));
	
		if (Strings)
		{
			QStrList  tmp = *Strings;
		
			if ( StringsDlg::EditStrings(tmp) )
			{
				 SetOrdValue( (unsigned long) &tmp );
			}  
		}
		return 0;
	}
};

PropEditorClass<TStringsProperty> _TStringsProperty("StrList");
RegisterClassEditor(QStrList, TStringsProperty);

/**********************************************************************
                    QMultiLineEdit::text  property editor
**********************************************************************/
class TStrings1Property : public TStringProperty
{
public:
	virtual int GetAttributes() { return paMultiSelect | paDialog; }
	virtual int Edit()
	{
		QString tmp = GetStrValue();
	
		if ( StringsDlg::EditStrings(tmp) )
		  SetStrValue( tmp );

		return 0;  
	}
};

PropEditorClass<TStrings1Property> _TStrings1Property("StrList");
RegisterPropertyEditor(QMultiLineEdit, text, TStrings1Property);




/*************************************************************************
		QPixmap class editor
*************************************************************************/
class TPixmapProperty : public TClassProperty
{
public:
	virtual int GetAttributes() { return paMultiSelect | paDialog; }
	virtual int Edit()
	{
		QString filename(QFileDialog::getOpenFileName(0, "*.xpm"));
		
		if (filename.length()>0)
		{
			QPixmap  pixmap(filename.data());
			SetOrdValue((unsigned int) &pixmap);
		}
		return 0;
 	}
 };

PropEditorClass<TPixmapProperty> _TPixmapProperty("Pixmap");
RegisterClassEditor(QPixmap, TPixmapProperty);



/*************************************************************************
		QMenuBar class editor
*************************************************************************/
#include "dlgs/menueditor.h"

/************************************************************************
	   QMenuItem::text  property editor
***********************************************************************/	   
class TTextProperty : public TStringProperty
{
public:
	virtual int GetAttributes() { return 0; }
};
PropEditorClass<TTextProperty> _TTextProperty("Text");

class TMenuDataProperty : public TClassProperty
{
public:
	virtual int GetAttributes() { return paDialog; }
	virtual int Edit()
	{
		QObject *obj = (QObject*) objectInspector->objs.obj(0);

		MenuEditor *dlg;
		if (obj->inherits("QPopupMenu"))
		dlg = 
		  new MenuEditor(objectInspector->designer, 
		  		 obj,
		                 (QPopupMenu*) obj, 
		                 true);
		else
		dlg = 
		  new MenuEditor(objectInspector->designer, 
		  		 obj,
		                 (QMenuBar*) obj, 
		                 false);

		dlg->show();
		
		return 0;
 	}
 };

PropEditorClass<TMenuDataProperty> _QMenuDataProperty("QMenuData");
RegisterClassEditor(QMenuBar, QMenuDataProperty);
RegisterClassEditor(QPopupMenu, QMenuDataProperty);
RegisterPropertyEditor(QMenuItem, text, TTextProperty);

#include <qtoolbar.h>
#include "toolbar.xpm"
class QToolBar1: public  QToolBar
{
public:
	QToolBar1(QWidget *P): QToolBar((QMainWindow*)P) {}
};

Designer *designerCreateQToolBar(Designer *p) \
{ return new Component<QToolBar1>(p, 0); }	\
static ComponentClass classQToolBar(ToolBar, "Qt", "qtoolbar.h", "QToolBar", designerCreateQToolBar);

